#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  merge_handler.py
#   Purpose:   handling merging mode in obspyDMT
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
import fnmatch
from obspy.core import read
import os

from event_handler import quake_info
from utility_codes import read_station_event

import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)

# ##################### FDSN_ARC_merge ##################################


def FDSN_ARC_merge(input_dics, clients):
    """
    Call "merge_stream" function that merges the retrieved waveforms in
    continuous request
    :param input_dics:
    :param clients:
    :return:
    """
    print '\n*****************************'
    print '%s -- Merging the waveforms' % clients
    print '*****************************'
    # Following two if-conditions create address to which merging
    # should be applied
    # Please note that these two conditions can not happen at the same time
    address = None
    network_name = None
    BH_file = None
    if clients == 'arc':
        clients_name = 'arc'
    else:
        clients_name = 'fdsn'
    if input_dics[clients_name + '_merge_auto'] == 'Y':
        period = '{0:s}_{1:s}'.\
            format(input_dics['min_date'].split('T')[0],
                   input_dics['max_date'].split('T')[0])
        address = os.path.join(input_dics['datapath'], period)

    if input_dics[clients_name + '_merge'] != 'N':
        address = input_dics[clients_name + '_merge']

    events, address_events = quake_info(address, 'info')

    ls_saved_stas_tmp = []
    for i in range(len(events)):
        sta_ev = read_station_event(address_events[i])
        # initialize some parameters which will be used later in merge_stream
        for s_ev in sta_ev[0]:
            if input_dics[clients_name + '_merge_auto'] == 'Y':
                if clients == s_ev[13]:
                    network, network_name, BH_file = \
                        init_merging(input_dics, s_ev[0])
                    station_id = '%s.%s.%s.%s' % (network, s_ev[1],
                                                  s_ev[2], s_ev[3])
                    ls_saved_stas_tmp.append(
                        os.path.join(address_events[i], BH_file, station_id))

            else:
                network, network_name, BH_file = \
                    init_merging(input_dics, s_ev[0])
                station_id = '%s.%s.%s.%s' % (network, s_ev[1],
                                              s_ev[2], s_ev[3])
                ls_saved_stas_tmp.append(
                    os.path.join(address_events[i], BH_file, station_id))

    if not input_dics['net'].startswith('_'):
        pattern_sta = '%s.%s.%s.%s' % (input_dics['net'], input_dics['sta'],
                                       input_dics['loc'], input_dics['cha'])
    else:
        pattern_sta = '*.%s.%s.%s' % (input_dics['sta'], input_dics['loc'],
                                      input_dics['cha'])

    ls_saved_stas = []
    for saved_sta in ls_saved_stas_tmp:
        if fnmatch.fnmatch(os.path.basename(saved_sta), pattern_sta):
            ls_saved_stas.append(saved_sta)

    if len(ls_saved_stas) != 0:
        saved_stations_names = []
        for ls_saved_sta in ls_saved_stas:
            saved_stations_names.append(os.path.basename(ls_saved_sta))
        ls_sta = list(set(saved_stations_names))

        ls_address = []
        for add_ev in address_events:
            ls_address.append(os.path.join(add_ev, BH_file))
        print 'Merging the waveforms...'
        merge_stream(input_dics=input_dics, ls_address=ls_address,
                     ls_sta=ls_sta, network_name=network_name)
        print 'Finish merging the waveforms'
    else:
        print "\nThere is no waveform to merge!"

# ##################### merge_stream ####################################


def merge_stream(input_dics, ls_address, ls_sta, network_name):
    """
    merges the waveforms in continuous requests
    Merging technique: (method=1)
    Discard data of the previous trace assuming the following trace
    contains data with a more correct time value.
    The parameter interpolation_samples specifies the number of samples used
    to linearly interpolate between
    the two traces in order to prevent steps.
    Note that if there are gaps inside, the returned array is still
    a masked array, only if fill_value is set,
    the returned array is a normal array and gaps are filled with fill value.
    No interpolation (interpolation_samples=0):

    Trace 1: AAAAAAAA
    Trace 2:     FFFFFFFF
    1 + 2  : AAAAFFFFFFFF

    :param input_dics:
    :param ls_address:
    :param ls_sta:
    :param network_name:
    :return:
    """
    address = os.path.dirname(os.path.dirname(ls_address[0]))

    try:
        os.makedirs(os.path.join(address, 'MERGED-%s' % network_name))
    except Exception as e:
        print "ERROR in creating a directory in %s" % address
        print e
        pass

    for sta in ls_sta:
        for j in range(len(ls_address)):
            if os.path.isfile(os.path.join(ls_address[j], sta)):
                st = read(os.path.join(ls_address[j], sta))
                for k in range(j+1, len(ls_address)):
                    try:
                        tr_tmp = read(os.path.join(ls_address[k], sta))
                        tr_tmp.merge(method=1, fill_value=0,
                                     interpolation_samples=0)
                        st.append(tr_tmp[0])
                    except Exception as e:
                        print "ERROR: can not append to the trace! \n%s" % e

                st.merge(method=1, fill_value=0, interpolation_samples=0)
                trace = st[0]
                trace_identity = '%s.%s.%s.%s' % (trace.stats['network'],
                                                  trace.stats['station'],
                                                  trace.stats['location'],
                                                  trace.stats['channel'])
                if input_dics['mseed'] == 'N':
                    st.write(os.path.join(address, 'MERGED-%s'
                                          % network_name, trace_identity),
                             format='SAC')
                else:
                    st.write(os.path.join(address, 'MERGED-%s'
                                          % network_name, trace_identity),
                             format='MSEED')
                break

# ##################### init_merging ####################################


def init_merging(input_dics, net_name):
    """
    Create required variables for merging
    """
    network_name = None
    network = None
    BH_file = None

    if input_dics['merge_type'] == 'raw':
        BH_file = 'BH_RAW'
        network = net_name
        network_name = 'raw'
    elif input_dics['merge_type'] == 'corrected':
        if input_dics['corr_unit'] == 'DIS':
            BH_file = 'BH'
            network = 'dis.%s' % net_name
            network_name = 'dis'
        elif input_dics['corr_unit'] == 'VEL':
            BH_file = 'BH_' + input_dics['corr_unit']
            network = 'vel.%s' % net_name
            network_name = 'vel'
        elif input_dics['corr_unit'] == 'ACC':
            BH_file = 'BH_' + input_dics['corr_unit']
            network = 'acc.%s' % net_name
            network_name = 'acc'
    return network, network_name, BH_file



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  fdsn_handler.py
#   Purpose:   handling FDSN in obspyDMT
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
from datetime import datetime
import fileinput
import glob
import multiprocessing
try:
    from obspy.clients.fdsn import Client as Client_fdsn
except Exception, e:
    from obspy.fdsn import Client as Client_fdsn
import os
import pickle

from event_handler import quake_info, create_folders_files
from format_converter import writesac_all
from utility_codes import read_list_stas, calculate_time_phase, getFolderSize

import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)

# ###################### FDSN_network ####################################


def FDSN_network(input_dics, events):
    """
    Availability and retrieivng functions are called with FDSN_network
    :param input_dics:
    :param events:
    :return:
    """
    print '\n********************************************************'
    print 'FDSN -- Download waveforms, StationXML files and meta-data'
    print '********************************************************'
    period = '{0:s}_{1:s}'.format(
        input_dics['min_date'].split('T')[0],
        input_dics['max_date'].split('T')[0])
    eventpath = os.path.join(input_dics['datapath'], period)

    print 'Initializing folders and files...',
    create_folders_files(events, eventpath, input_dics)
    print 'DONE'

    for i in range(len(events)):
        t_fdsn_1 = datetime.now()
        target_path = os.path.join(eventpath, events[i]['event_id'])

        if not input_dics['list_stas']:
            Stas_fdsn = FDSN_available(input_dics, events[i],
                                       target_path, event_number=i)
        else:
            Stas_fdsn = read_list_stas(input_dics['list_stas'],
                                       input_dics['normal_mode_syn'],
                                       input_dics['specfem3D'])
            if input_dics['fdsn_bulk'] == 'Y':
                FDSN_create_bulk_list(target_path, input_dics, Stas_fdsn,
                                      events[i])

        if input_dics['fdsn_bulk'] != 'Y':
            print '\n%s-Availability for event: %s/%s ---> DONE' \
                  % (input_dics['fdsn_base_url'], i+1, len(events))
        else:
            print '\nFDSN-bulkfile for event: %s/%s ---> DONE' \
                  % (i+1, len(events))

        print 'Time for checking the availability: %s' \
              % (datetime.now() - t_fdsn_1)

        if Stas_fdsn != [[]]:
            FDSN_waveform(input_dics, events, Stas_fdsn, i, req_type='save')
        else:
            print 'No available station in %s for your request and ' \
                  'for event %s!' % (input_dics['fdsn_base_url'], str(i+1))
            continue

# ##################### FDSN_available ##################################


def FDSN_available(input_dics, event, target_path, event_number):
    """
    Check the availablity of FDSN stations
    :param input_dics:
    :param event:
    :param target_path:
    :param event_number:
    :return:
    """
    print "Check the availablity of FDSN stations: %s" \
          % input_dics['fdsn_base_url']
    client_fdsn = Client_fdsn(base_url=input_dics['fdsn_base_url'],
                              user=input_dics['fdsn_user'],
                              password=input_dics['fdsn_pass'])
    Sta_fdsn = []
    try:
        if input_dics['fdsn_base_url'].lower() in ['resif']:
            msg = 'WARNING\n' \
                  'You have selected RESIF as fdsn_base_url!\n\n' \
                  'I have detected some problems in sending request to ' \
                  'this data provider:\n' \
                  'It seems that the start and end times in channel level ' \
                  'are not set correctly.\n\n' \
                  'It is better to use --list_stas options other ' \
                  'than sending the availability\nrequest directly.'
            print '\n\n' + 80*'='
            print msg
            print 80*'=' + '\n\n'
            # start_time = None
            # end_time = None
            start_time = event['t1']
            end_time = event['t2']
        else:
            start_time = event['t1']
            end_time = event['t2']
        available = client_fdsn.get_stations(
            network=input_dics['net'],
            station=input_dics['sta'],
            location=input_dics['loc'],
            channel=input_dics['cha'],
            starttime=start_time,
            endtime=end_time,
            latitude=input_dics['lat_cba'],
            longitude=input_dics['lon_cba'],
            minradius=input_dics['mr_cba'],
            maxradius=input_dics['Mr_cba'],
            minlatitude=input_dics['mlat_rbb'],
            maxlatitude=input_dics['Mlat_rbb'],
            minlongitude=input_dics['mlon_rbb'],
            maxlongitude=input_dics['Mlon_rbb'],
            level='channel')

        for network in available.networks:
            for station in network:
                for channel in station:
                    Sta_fdsn.append([network.code, station.code,
                                     channel.location_code, channel.code,
                                     channel.latitude, channel.longitude,
                                     channel.elevation, channel.depth])
        if input_dics['fdsn_bulk'] == 'Y':
            if input_dics['fdsn_update'] != 'N':
                if os.path.exists(os.path.join(target_path, 'info',
                                               'bulkdata.txt')):
                    os.remove(os.path.join(target_path, 'info',
                                           'bulkdata.txt'))
            if os.path.exists(os.path.join(target_path, 'info',
                                           'bulkdata.txt')):
                print 'bulkdata.txt exists in the directory!'
            else:
                print 'Start creating a list for bulk request'
                bulk_list = []
                for bulk_sta in Sta_fdsn:
                    if input_dics['cut_time_phase']:
                        t_start, t_end = calculate_time_phase(event, bulk_sta)
                    else:
                        t_start = event['t1']
                        t_end = event['t2']
                    bulk_list.append((bulk_sta[0], bulk_sta[1], bulk_sta[2],
                                      bulk_sta[3], t_start, t_end))

                bulk_list_fio = open(os.path.join(target_path, 'info',
                                                  'bulkdata_list'), 'a+')
                pickle.dump(bulk_list, bulk_list_fio)
                bulk_list_fio.close()
    except Exception as e:
        exc_file = open(os.path.join(target_path, 'info', 'exception'), 'a+')
        ee = 'fdsn -- Event: %s --- %s\n' % (str(event_number+1), e)
        exc_file.writelines(ee)
        exc_file.close()
        print 'ERROR: %s' % ee

    if len(Sta_fdsn) == 0:
        Sta_fdsn.append([])
    Sta_fdsn.sort()
    return Sta_fdsn

# ##################### FDSN_create_bulk_list ###############################


def FDSN_create_bulk_list(target_path, input_dics, Stas_fdsn, event):
    """
    Create bulkdata_list in case of --list_stas flag
    :param target_path:
    :param input_dics:
    :param Stas_fdsn:
    :param event:
    :return:
    """
    if input_dics['fdsn_update'] != 'N':
        if os.path.exists(os.path.join(target_path, 'info', 'bulkdata.txt')):
            os.remove(os.path.join(target_path, 'info', 'bulkdata.txt'))

    if os.path.exists(os.path.join(target_path, 'info', 'bulkdata.txt')):
        print 'bulkdata.txt exists in the directory!'
    else:
        print 'Start creating a list for bulk request'
        bulk_list = []
        for bulk_sta in Stas_fdsn:
            if input_dics['cut_time_phase']:
                t_start, t_end = calculate_time_phase(event, bulk_sta)
            else:
                t_start = event['t1']
                t_end = event['t2']
            bulk_list.append((bulk_sta[0], bulk_sta[1], bulk_sta[2],
                              bulk_sta[3], t_start, t_end))

        bulk_list_fio = open(os.path.join(target_path, 'info',
                                          'bulkdata_list'), 'a+')
        pickle.dump(bulk_list, bulk_list_fio)
        bulk_list_fio.close()

# ##################### FDSN_waveform ###############################


def FDSN_waveform(input_dics, events, Sta_req, i, req_type):
    """
    Gets Waveforms, StationXML files and meta-data from FDSN
    :param input_dics:
    :param events:
    :param Sta_req:
    :param i:
    :param req_type:
    :return:
    """
    t_wave_1 = datetime.now()

    add_event = []
    if req_type == 'save':
        period = '{0:s}_{1:s}'.\
            format(input_dics['min_date'].split('T')[0],
                   input_dics['max_date'].split('T')[0])
        eventpath = os.path.join(input_dics['datapath'], period)
        for k in range(len(events)):
            add_event.append(os.path.join(eventpath, events[k]['event_id']))
            events_fio = open(os.path.join(add_event[k],
                                           'info', 'event.pkl'), 'w')
            pickle.dump(events[k], events_fio)
            events_fio.close()
    elif req_type == 'update':
        events, add_event = \
            quake_info(input_dics['fdsn_update'], target='info')

    if input_dics['test'] == 'Y':
        len_req_fdsn = input_dics['test_num']
    else:
        len_req_fdsn = len(Sta_req)

    if input_dics['fdsn_bulk'] == 'Y':
        t11 = datetime.now()
        try:
            FDSN_bulk_request(i, add_event, input_dics)
        except Exception as e:
            print 'WARNING: %s' % e
        print 'DONE'

        # Following parameter is set to 'N' to avoid
        # retrieving the waveforms twice
        # When using bulk requests, waveforms are retreived in bulk
        # but not response/StationXML files and not metadata
        input_dics['waveform'] = 'N'
        t22 = datetime.now()
        print '\nbulkdataselect request is done for event: %s/%s in %s' \
              % (i+1, len(events), t22-t11)

    FDSN_serial_parallel(i, events, add_event, Sta_req, input_dics,
                         len_req_fdsn)
    if input_dics['SAC'] == 'Y':
        print '\nConverting the MSEED files to SAC...',
        writesac_all(i=i, address_events=add_event)
        print 'DONE'

    try:
        len_sta_ev_open = open(os.path.join(add_event[i], 'info',
                                            'station_event'), 'r')
        len_sta_ev = len(len_sta_ev_open.readlines())
    except IOError:
        len_sta_ev = 'Can not open station_event file: %s' \
                     % (os.path.join(add_event[i], 'info', 'station_event'))

    FDSN_reporter(i, add_event, events, input_dics, Sta_req, len_sta_ev,
                  req_type, t_wave_1)

# ##################### FDSN_serial_parallel ##################################


def FDSN_serial_parallel(i, events, add_event, Sta_req, input_dics,
                         len_req_fdsn):
    """
    FDSN serial/parallel request
    """
    dic = {}
    print '\nFDSN-Event: %s/%s -- %s' % (i+1, len(events), add_event[i])

    client_fdsn = Client_fdsn(base_url=input_dics['fdsn_base_url'],
                              user=input_dics['fdsn_user'],
                              password=input_dics['fdsn_pass'])
    if input_dics['req_parallel'] == 'Y':
        print "Parallel request with %s processes.\n" % input_dics['req_np']
        par_jobs = []
        for j in range(len_req_fdsn):
            p = multiprocessing.Process(target=FDSN_download_core,
                                        args=(i, j, dic, len(events),
                                              events, add_event, Sta_req,
                                              input_dics, client_fdsn,))
            par_jobs.append(p)
        sub_par_jobs = []
        for l in range(len(par_jobs)):
            counter = input_dics['req_np']
            while counter >= input_dics['req_np']:
                counter = 0
                for ll in range(len(sub_par_jobs)):
                    if par_jobs[sub_par_jobs[ll]].is_alive():
                        counter += 1
            par_jobs[l].start()
            sub_par_jobs.append(l)

        counter = input_dics['req_np']
        while counter > 0:
            counter = 0
            for ll in range(len(par_jobs)):
                if par_jobs[ll].is_alive():
                    counter += 1
    else:
        for j in range(len_req_fdsn):
            FDSN_download_core(i=i, j=j, dic=dic, len_events=len(events),
                               events=events, add_event=add_event,
                               Sta_req=Sta_req, input_dics=input_dics,
                               client_fdsn=client_fdsn)

    if input_dics['fdsn_bulk'] == 'Y':
        input_dics['waveform'] = 'Y'
        sta_saved_path = glob.glob(os.path.join(add_event[i],
                                                'BH_RAW',
                                                '*.*.*.*'))
        print '\nAdjusting the station_event file...',

        sta_saved_list = []
        for sta_num in range(len(sta_saved_path)):
            sta_saved_list.append(os.path.basename(sta_saved_path[sta_num]))

        sta_ev_new = []
        for line in fileinput.FileInput(
                os.path.join(add_event[i], 'info', 'station_event')):
            line_split = line.split(',')
            if not '%s.%s.%s.%s' \
                    % (line_split[0], line_split[1], line_split[2],
                       line_split[3]) in sta_saved_list:
                pass
            else:
                sta_ev_new.append(line)

        file_staev_open = open(os.path.join(add_event[i], 'info',
                                            'station_event'), 'w')
        file_staev_open.writelines(sta_ev_new)
        file_staev_open.close()
        print 'DONE'

# ##################### FDSN_download_core ##################################


def FDSN_download_core(i, j, dic, len_events, events, add_event,
                       Sta_req, input_dics, client_fdsn):
    """
    Downloading the waveforms, reponse files (StationXML) and metadata
    This program should be normally called by some higher-level functions
    """
    dummy = 'Initializing'
    info_req = 'None'
    t11 = datetime.now()
    try:
        info_req = '[%s/%s-%s/%s-%s] ' % (i+1, len_events, j+1,
                                          len(Sta_req), input_dics['cha'])

        if Sta_req[j][2] == '--' or Sta_req[j][2] == '  ':
                Sta_req[j][2] = ''

        if input_dics['cut_time_phase']:
            t_start, t_end = calculate_time_phase(events[i], Sta_req[j])
        else:
            t_start = events[i]['t1']
            t_end = events[i]['t2']

        if input_dics['waveform'] == 'Y':
            dummy = 'Waveform'
            client_fdsn.get_waveforms(Sta_req[j][0], Sta_req[j][1],
                                      Sta_req[j][2], Sta_req[j][3],
                                      t_start, t_end,
                                      filename=os.path.join(add_event[i],
                                                            'BH_RAW',
                                                            '%s.%s.%s.%s'
                                                            % (Sta_req[j][0],
                                                               Sta_req[j][1],
                                                               Sta_req[j][2],
                                                               Sta_req[j][3])))
            print '%ssaving waveform for: %s.%s.%s.%s  ---> DONE' \
                  % (info_req, Sta_req[j][0], Sta_req[j][1],
                     Sta_req[j][2], Sta_req[j][3])

        if input_dics['response'] == 'Y':
            dummy = 'Response'
            client_fdsn.get_stations(network=Sta_req[j][0],
                                     station=Sta_req[j][1],
                                     location=Sta_req[j][2],
                                     channel=Sta_req[j][3],
                                     # starttime=t_start, endtime=t_end,
                                     filename=os.path.join(add_event[i],
                                                           'Resp',
                                                           'STXML.%s.%s.%s.%s'
                                                           % (Sta_req[j][0],
                                                              Sta_req[j][1],
                                                              Sta_req[j][2],
                                                              Sta_req[j][3])),
                                     level='response')

            print "%ssaving Response for: %s.%s.%s.%s  ---> DONE" \
                  % (info_req, Sta_req[j][0], Sta_req[j][1],
                     Sta_req[j][2], Sta_req[j][3])

        dummy = 'Meta-data'
        dic[j] = {'info': '%s.%s.%s.%s' % (Sta_req[j][0], Sta_req[j][1],
                                           Sta_req[j][2], Sta_req[j][3]),
                  'net': Sta_req[j][0],
                  'sta': Sta_req[j][1],
                  'latitude': Sta_req[j][4],
                  'longitude': Sta_req[j][5],
                  'loc': Sta_req[j][2],
                  'cha': Sta_req[j][3],
                  'elevation': Sta_req[j][6],
                  'depth': Sta_req[j][7]}
        Syn_file = open(os.path.join(add_event[i], 'info',
                                     'station_event'), 'a')
        syn = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,\n' \
              % (dic[j]['net'], dic[j]['sta'],
                 dic[j]['loc'], dic[j]['cha'],
                 dic[j]['latitude'], dic[j]['longitude'],
                 float(dic[j]['elevation']),
                 float(dic[j]['depth']),
                 events[i]['event_id'], events[i]['latitude'],
                 events[i]['longitude'], events[i]['depth'],
                 events[i]['magnitude'], input_dics['fdsn_base_url'])
        Syn_file.writelines(syn)
        Syn_file.close()

        print "%ssaving Metadata for: %s.%s.%s.%s  ---> DONE" \
              % (info_req, Sta_req[j][0], Sta_req[j][1],
                 Sta_req[j][2], Sta_req[j][3])

        t22 = datetime.now()
        if input_dics['time_fdsn'] == 'Y':
            time_fdsn = t22 - t11
            time_file = open(os.path.join(add_event[i], 'info',
                                          'time_fdsn'), 'a')
            size = getFolderSize(os.path.join(add_event[i]))
            ti = '%s,%s,%s,%s,%s,%s,%s,+,\n' % (Sta_req[j][0],
                                                Sta_req[j][1],
                                                Sta_req[j][2],
                                                Sta_req[j][3],
                                                time_fdsn.seconds,
                                                time_fdsn.microseconds,
                                                size/(1024.**2))
            time_file.writelines(ti)
            time_file.close()
    except Exception as e:
        t22 = datetime.now()
        if input_dics['time_fdsn'] == 'Y':
            time_fdsn = t22 - t11
            time_file = open(os.path.join(add_event[i], 'info',
                                          'time_fdsn'), 'a')
            size = getFolderSize(os.path.join(add_event[i]))
            ti = '%s,%s,%s,%s,%s,%s,%s,-,\n' % (Sta_req[j][0],
                                                Sta_req[j][1],
                                                Sta_req[j][2],
                                                Sta_req[j][3],
                                                time_fdsn.seconds,
                                                time_fdsn.microseconds,
                                                size/(1024.**2))
            time_file.writelines(ti)
            time_file.close()

        if len(Sta_req[j]) != 0:
            print '%s%s---%s.%s.%s.%s' % (info_req, dummy,
                                          Sta_req[j][0],
                                          Sta_req[j][1],
                                          Sta_req[j][2],
                                          Sta_req[j][3])
            ee = 'fdsn -- %s---%s-%s---%s.%s.%s.%s---%s\n' \
                 % (dummy, i+1, j+1,
                    Sta_req[j][0], Sta_req[j][1], Sta_req[j][2],
                    Sta_req[j][3], e)
        else:
            ee = 'There is no available station for this event.'
        Exception_file = open(os.path.join(add_event[i],
                                           'info', 'exception'), 'a')
        Exception_file.writelines(ee)
        Exception_file.close()
        print 'ERROR: %s' % ee

# ##################### FDSN_bulk_request ##################################


def FDSN_bulk_request(i, add_event, input_dics):
    """
    Send bulk request to FDSN
    """
    print '\nSending bulk request to FDSN: %s' % input_dics['fdsn_base_url']
    client_fdsn = Client_fdsn(base_url=input_dics['fdsn_base_url'],
                              user=input_dics['fdsn_user'],
                              password=input_dics['fdsn_pass'])
    bulk_list_fio = open(os.path.join(add_event[i], 'info',
                                      'bulkdata_list'))
    bulk_list = pickle.load(bulk_list_fio)
    bulk_smgrs = client_fdsn.get_waveforms_bulk(bulk_list)
    print 'Saving the retrieved waveforms...',
    for bulk_st in bulk_smgrs:
        bulk_st.write(os.path.join(add_event[i], 'BH_RAW',
                                   '%s.%s.%s.%s'
                                   % (bulk_st.stats['network'],
                                      bulk_st.stats['station'],
                                      bulk_st.stats['location'],
                                      bulk_st.stats['channel'])),
                      'MSEED')

# ##################### FDSN_reporter ##################################


def FDSN_reporter(i, add_event, events, input_dics, Sta_req, len_sta_ev,
                  req_type, t_wave_1):
    """
    Writing reports for the request
    """
    report = open(os.path.join(add_event[i], 'info', 'report_st'), 'a')
    eventsID = events[i]['event_id']
    report.writelines('<><><><><><><><><><><><><><><><><>\n')
    report.writelines(eventsID + '\n')
    report.writelines('---------------%s---------------\n'
                      % input_dics['fdsn_base_url'])
    report.writelines('---------------%s---------------\n' % input_dics['cha'])
    rep = 'FDSN-Available stations for channel %s and for event-%s: %s\n' \
          % (input_dics['cha'], i, len(Sta_req))
    report.writelines(rep)
    rep = 'FDSN-%s stations for channel %s and for event-%s: %s\n' \
          % (req_type, input_dics['cha'], i, len_sta_ev)
    report.writelines(rep)
    report.writelines('----------------------------------\n')

    t_wave = datetime.now() - t_wave_1

    rep = 'Time for %sing Waveforms from FDSN: %s\n' % (req_type, t_wave)
    report.writelines(rep)
    report.writelines('----------------------------------\n')
    report.close()

    if input_dics['req_parallel'] == 'Y':
        report_parallel_open = open(os.path.join(add_event[i], 'info',
                                                 'report_parallel'), 'a')
        report_parallel_open.writelines(
            '---------------%s---------------\n'
            % input_dics['fdsn_base_url'])
        report_parallel_open.writelines('Request\n')
        if input_dics['fdsn_bulk'] == 'Y':
            report_parallel_open.writelines('Number of Nodes: (bulk) %s\n'
                                            % input_dics['req_np'])
        else:
            report_parallel_open.writelines('Number of Nodes: %s\n'
                                            % input_dics['req_np'])

        size = getFolderSize(os.path.join(add_event[i]))
        ti = '%s,%s,%s,+,\n' % (t_wave.seconds,
                                t_wave.microseconds,
                                size/(1024.**2))

        report_parallel_open.writelines('Total Time     : %s\n' % t_wave)
        report_parallel_open.writelines(ti)
        report_parallel_open.close()

    print "\n------------------------"
    print 'FDSN for event-%s is Done' % (i+1)
    print 'Total Time: %s' % t_wave
    print "------------------------"


# -------------------------------- TRASH
# parallel_len_req_fdsn = range(0, len_req_fdsn)

# start = 0
# end = len_req_fdsn
# step = (end - start) / input_dics['req_np'] + 1

# jobs = []
# for index in xrange(input_dics['req_np']):
#     starti = start+index*step
#     endi = min(start+(index+1)*step, end)
#     p = multiprocessing.Process(target=FDSN_download_iter,
#                                 args=(i, starti, endi, dic, req_type,
#                                       len(events), events, add_event,
#                                       Sta_req, input_dics, client_fdsn))

#     jobs.append(p)
# for index in range(len(jobs)):
#     jobs[index].start()

# pp_flag = True
# while pp_flag:
#     for proc in jobs:
#         if proc.is_alive():
#             time.sleep(1)
#             pp_flag = True
#             break
#         else:
#             pp_flag = False
#     if not pp_flag:
#         print '\nAll the processes are finished...'

# len_par_grp = [parallel_len_req_fdsn[n:n+input_dics['req_np']] for n in
#                range(0, len(parallel_len_req_fdsn), input_dics['req_np'])]
# # ##################### FDSN_download_iter ##################################
#
#
# def FDSN_download_iter(i, starti, endi, dic, type, len_events, events,
#                        add_event, Sta_req, input, client_fdsn):
#     """
#     This function only iterates over FDSN_download_core,
#     this should be called by another program.
#     """
#     for j in range(starti, endi):
#         FDSN_download_core(i=i, j=j, dic=dic, type=type,
#                            len_events=len_events, events=events,
#                            add_event=add_event, Sta_req=Sta_req,
#                            input=input, client_fdsn=client_fdsn)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  event_handler.py
#   Purpose:   handling events in obspyDMT
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
from collections import OrderedDict
import copy
from datetime import datetime, timedelta
import glob
import matplotlib.pyplot as plt
import numpy as np
from obspy.core.event import Catalog, readEvents
from obspy.core import read, UTCDateTime
try:
    from obspy.geodetics import locations2degrees
except Exception, e:
    from obspy.core.util import locations2degrees
try:
    from obspy.clients.fdsn import Client as Client_fdsn
except Exception, e:
    from obspy.fdsn import Client as Client_fdsn
import os
import pickle
import re
import shutil
import sys
import tarfile
import time
import urllib2

from input_handler import input_logger
from utility_codes import locate

import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)

# ##################### get_Events ######################################


def get_Events(input_dics, request):
    """
    Generating list of events
    :param input_dics:
    :param request:
    :return:
    """
    t_event_1 = time.time()

    # request can be 'event-based' or continuous
    try:
        events, catalog, successful_read = events_info(input_dics, request)
    except Exception, e:
        print 'WARNING: %s' % e
        return 0

    if len(events) < 1:
        if not input_dics['plot_all_events']:
            print "ERROR: no event available to proceed"
        return 0

    events, catalog, events2, row_format, header, input_dics = \
        output_shell_event(input_dics, events, catalog, successful_read,
                           request)

    print 'Number of event(s): %s' % len(events)
    print 'Time for retrieving and saving the event info: %s' \
          % str(timedelta(seconds=round(float(time.time() - t_event_1))))

    # formatting output / check if directory exists
    period = '{0:s}_{1:s}'.format(
        input_dics['min_date'].split('T')[0],
        input_dics['max_date'].split('T')[0])
    eventpath = os.path.join(input_dics['datapath'], period)

    write_cat_logger(input_dics, eventpath, period, events, catalog,
                     events2, row_format, header)
    return events

# ##################### events_info #####################################


def events_info(input_dics, request):
    """
    Get event(s) info for event-based or continuous requests
    :param input_dics:
    :param request:
    :return:
    """
    successful_read = 0

    events = []
    events_QML = []
    if request == 'event-based':
        try:
            # if: read event file ; else make url request for events
            if input_dics['read_catalog'] != 'N':

                if input_dics['event_catalog']:
                    print '\n\033[91mContradictory options chosen:\033[0m\nYou '\
                          'specifed a catalog by "--event_catalog"\nand by "--read'\
                          '_catalog".\nProceed with reading file !'

                try:
                    events_QML = readEvents(input_dics['read_catalog'])
                    print '\nRead events from catalog:\n' \
                          '>>:\t%s' % input_dics['read_catalog']
                    successful_read = 1
                except TypeError as err:
                    print 'WARNING: %s' % err
                    print '\n\033[91mData format not supported,\033[0m' \
                          ' tried to read file:\n>>:\t', \
                        input_dics['read_catalog']
                    print '\nFor valid data formats, see:'
                    print 'obspyDMT -h | grep read_catalog'
                    return
                except IOError as err:
                    print '\n\033[91mStated file does not exist:\033[0m\n' \
                          '>>:\t', err
                    return
                except Exception as err:
                    print '\n\033[91mSomething went terribly wrong:\033[0m\n' \
                          '>>:\t', err
                    return

            else:

                evlatmin = input_dics['evlatmin']
                evlatmax = input_dics['evlatmax']
                evlonmin = input_dics['evlonmin']
                evlonmax = input_dics['evlonmax']

                evlat = input_dics['evlat']
                evlon = input_dics['evlon']
                evradmax = input_dics['evradmax']
                evradmin = input_dics['evradmin']

                event_switch = 'fdsn'
                if input_dics['event_catalog']:
                    if input_dics['event_catalog'].lower() == 'gcmt_combo':
                        event_switch = 'gcmt_combo'
                    if input_dics['event_catalog'].lower() == 'neic_usgs':
                        event_switch = 'neic_usgs'
                    else:
                        print 'Event(s) are based on:\t',
                        print input_dics['event_url']
                else:
                    print 'Event(s) are based on:\t',
                    print input_dics['event_url']

                print 'Specified catalog: \t%s\n' % input_dics['event_catalog']

                if event_switch == 'fdsn':
                    client_fdsn = Client_fdsn(base_url=input_dics['event_url'])
                    events_QML = client_fdsn.get_events(
                        minlatitude=evlatmin,
                        maxlatitude=evlatmax,
                        minlongitude=evlonmin,
                        maxlongitude=evlonmax,
                        latitude=evlat,
                        longitude=evlon,
                        maxradius=evradmax,
                        minradius=evradmin,
                        mindepth=input_dics['min_depth'],
                        maxdepth=input_dics['max_depth'],
                        starttime=input_dics['min_date'],
                        endtime=input_dics['max_date'],
                        minmagnitude=input_dics['min_mag'],
                        maxmagnitude=input_dics['max_mag'],
                        orderby='time',
                        catalog=input_dics['event_catalog'],
                        magnitudetype=input_dics['mag_type'])
                elif event_switch == 'gcmt_combo':
                    events_QML = \
                        gcmt_catalog(input_dics['min_date'],
                                     input_dics['max_date'],
                                     evlatmin, evlatmax, evlonmin, evlonmax,
                                     evlat, evlon, evradmin, evradmax,
                                     input_dics['min_depth'],
                                     input_dics['max_depth'],
                                     input_dics['min_mag'],
                                     input_dics['max_mag'])
                elif event_switch == 'neic_usgs':
                    events_QML = \
                        neic_catalog(input_dics['min_date'],
                                     input_dics['max_date'],
                                     evlatmin, evlatmax, evlonmin, evlonmax,
                                     evlat, evlon, evradmin, evradmax,
                                     input_dics['min_depth'],
                                     input_dics['max_depth'],
                                     input_dics['min_mag'],
                                     input_dics['max_mag'])
                else:
                    sys.exit('%s is not supported'
                             % input_dics['event_catalog'])

            # no matter if list was passed or requested, sort catalogue,
            # plot events and proceed
            events_QML = sort_catalogue(events_QML)

            if input_dics['plot_all_events']:
                plt.ion()
                events_QML.plot()

            events = []
            for i in range(len(events_QML)):
                try:
                    event_time = \
                        events_QML.events[i].preferred_origin().time or \
                        events_QML.events[i].origins[0].time
                    event_time_month = '%02i' % int(event_time.month)
                    event_time_day = '%02i' % int(event_time.day)
                except Exception, e:
                    print e
                    continue

                try:
                    # XXX Check about the preferred?
                    if not events_QML.events[i].focal_mechanisms == []:
                        focal_mechanism = [
                            events_QML.events[i].preferred_focal_mechanism()
                            ['moment_tensor']['tensor']['m_rr'],
                            events_QML.events[i].preferred_focal_mechanism()
                            ['moment_tensor']['tensor']['m_tt'],
                            events_QML.events[i].preferred_focal_mechanism()
                            ['moment_tensor']['tensor']['m_pp'],
                            events_QML.events[i].preferred_focal_mechanism()
                            ['moment_tensor']['tensor']['m_rt'],
                            events_QML.events[i].preferred_focal_mechanism()
                            ['moment_tensor']['tensor']['m_rp'],
                            events_QML.events[i].preferred_focal_mechanism()
                            ['moment_tensor']['tensor']['m_tp']]
                    else:
                        focal_mechanism = False
                except AttributeError:
                    print "WARNING: focal_mechanism does not exist for " \
                          "event: %s -- set to False" % (i+1)
                    focal_mechanism = False
                except TypeError:
                    focal_mechanism = False
                except Exception, e:
                    print e
                    focal_mechanism = False
                try:
                    if not events_QML.events[i].focal_mechanisms == []:
                        half_duration = [
                            events_QML.events[i].preferred_focal_mechanism()
                            ['moment_tensor']['source_time_function']['type'],
                            events_QML.events[i].preferred_focal_mechanism()
                            ['moment_tensor']['source_time_function']
                            ['duration']]
                    else:
                        half_duration = mag_halfduration(
                            mag=events_QML.events[i].preferred_magnitude().mag)
                except AttributeError:
                    print "WARNING: half_duration does not exist for " \
                          "event: %s -- set to False" % (i+1)
                    half_duration = False
                except TypeError:
                    half_duration = False
                except Exception, e:
                    print e
                    focal_mechanism = False

                try:
                    events.append(OrderedDict(
                        [('number', i+1),
                         ('latitude',
                          events_QML.events[i].preferred_origin().latitude or
                          events_QML.events[i].origins[0].latitude),
                         ('longitude',
                          events_QML.events[i].preferred_origin().longitude or
                          events_QML.events[i].origins[0].longitude),
                         ('depth',
                          events_QML.events[i].preferred_origin().depth/1000. or
                          events_QML.events[i].origins[0].depth/1000.),
                         ('datetime', event_time),
                         ('magnitude',
                          events_QML.events[i].preferred_magnitude().mag or
                          events_QML.events[i].magnitudes[0].mag),
                         ('magnitude_type',
                          events_QML.events[i].preferred_magnitude().
                          magnitude_type or
                          events_QML.events[i].magnitudes[0].
                          magnitude_type),
                         ('author',
                          events_QML.events[i].preferred_magnitude().
                          creation_info.author or
                          events_QML.events[i].magnitudes[0].creation_info.author),
                         ('event_id', str(event_time.year) +
                          event_time_month + event_time_day + '_' + str(i+1)),
                         ('origin_id', events_QML.events[i].preferred_origin_id or
                          events_QML.events[i].origins[0].resource_id.resource_id),
                         ('focal_mechanism', focal_mechanism),
                         ('half_duration', half_duration),
                         ('flynn_region', 'NAN'),
                         ]))
                except Exception, e:
                    print e
                    continue

                # if --read_catalog, redefine variables which determine
                # the name of the folder where results will be stored. (Other-
                # wise folder is named after defaults (10 & 5 days ago)). For
                # renaming use max and min from quakes of read catalog
                if input_dics['read_catalog'] != 'N' and successful_read == 1:
                    input_dics['max_date'] = \
                        str(max([e['datetime'] for e in events]))
                    input_dics['min_date'] = \
                        str(min([e['datetime'] for e in events]))
                    input_dics['max_mag'] = \
                        str(max([e['magnitude'] for e in events]))
                    input_dics['min_mag'] = \
                        str(min([e['magnitude'] for e in events]))

        except Exception as e:
            print 60*'-'
            print 'ERROR: %s' % e
            print 60*'-'
            events = []
            events_QML = Catalog(events=[])

        for i in range(len(events)):
            events[i]['t1'] = events[i]['datetime'] - input_dics['preset']
            events[i]['t2'] = events[i]['datetime'] + input_dics['offset']

    elif request == 'continuous':

        if input_dics['read_catalog'] != 'N':
            print '\n\033[91mContradictory options chosen:\033[0m\nYou ' \
                  'specifed "--continuous"\ndata request and "--read_catalog".'\
                  '\nProceed with continuous data request !\n'
            print '-------------------------------------------------'

        print 'Start identifying the intervals ..'
        m_date = UTCDateTime(input_dics['min_date'])
        M_date = UTCDateTime(input_dics['max_date'])
        t_cont = M_date - m_date

        events = []
        if t_cont > input_dics['interval']:
            num_div = int(t_cont/input_dics['interval'])
            # residual time is: (has not been used here)
            # t_res = t_cont - num_div*input_dics['interval']
            for i in range(1, num_div+1):
                cont_dir_name = (len(str(num_div)) - len(str(i)))*'0' + str(i)
                events.append(OrderedDict(
                    [('number', i),
                     ('latitude', -12345),
                     ('longitude', -12345),
                     ('depth', -12345),
                     ('datetime', m_date + (i-1)*input_dics['interval']),
                     ('magnitude', -12345),
                     ('magnitude_type', 'NAN'),
                     ('author', 'NAN'),
                     ('event_id', 'continuous' + cont_dir_name),
                     ('origin_id', -12345),
                     ('focal_mechanism', False),
                     ('half_duration', False),
                     ('flynn_region', 'NAN'),
                     ('t1', m_date + (i-1)*input_dics['interval'] +
                      input_dics['preset_cont']),
                     ('t2', m_date + i*input_dics['interval'] +
                      input_dics['offset_cont']),
                     ]))

            final_time = m_date + num_div*input_dics['interval'] + \
                         input_dics['offset_cont']
            if not M_date == final_time:
                cont_dir_name = str(num_div+1)
                events.append(OrderedDict(
                    [('number', num_div+1),
                     ('latitude', -12345),
                     ('longitude', -12345),
                     ('depth', -12345),
                     ('datetime', m_date + num_div*input_dics['interval']),
                     ('magnitude', -12345),
                     ('magnitude_type', 'NAN'),
                     ('author', 'NAN'),
                     ('event_id', 'continuous' + cont_dir_name),
                     ('origin_id', -12345),
                     ('focal_mechanism', False),
                     ('half_duration', False),
                     ('flynn_region', 'NAN'),
                     ('t1', m_date + num_div*input_dics['interval'] +
                      input_dics['preset_cont']),
                     ('t2', M_date),
                     ]))
        else:
            events.append(OrderedDict(
                [('number', 1),
                 ('latitude', -12345),
                 ('longitude', -12345),
                 ('depth', -12345),
                 ('datetime', m_date),
                 ('magnitude', -12345),
                 ('magnitude_type', 'NAN'),
                 ('author', 'NAN'),
                 ('event_id', 'continuous1'),
                 ('origin_id', -12345),
                 ('focal_mechanism', False),
                 ('half_duration', False),
                 ('flynn_region', 'NAN'),
                 ('t1', m_date),
                 ('t2', M_date),
                 ]))
        print 'DONE'
        events_QML = Catalog(events=[])
    return events, events_QML, successful_read

# ##################### output_shell_event ####################################


def output_shell_event(input_dics, events, catalog, successful_read, request):
    """
    output event information to the shell and remove those that are
    selected by the user
    :param input_dics:
    :param events:
    :param catalog:
    :param successful_read:
    :param request:
    :return:
    """
    # output shell
    spaces, events2, header = event_spaces(events=events, request=request)
    header_template = ['{:<'+str(e+2)+'}' for e in spaces]
    row_format = '{}'.format(''.join(header_template))

    if input_dics['user_select_event'] == 'Y':
        print '\n' + row_format.format(*header)
        print 80 * '-'
        for i in range(len(events2)):
            print (row_format.format(*events2[i].values())).rstrip()
        print 80 * '-' + '\n'
        events, catalog, garbage = \
            delete_events(events=events, catalog=catalog)

    else:
        garbage = []
        if len(events) == 0:
            pass

        elif len(events) <= 50:
            print '\n' + row_format.format(*header)
            print 80 * '-'
            for i in range(len(events2)):
                print (row_format.format(*events2[i].values())).rstrip()
            print 80 * '-' + '\n'

        else:
            print '\n' + row_format.format(*header)
            print 80 * '-'
            print (row_format.format(*events2[0].values())).rstrip()
            print (row_format.format(*events2[1].values())).rstrip()
            print (row_format.format(*events2[2].values())).rstrip()
            print '..'
            print (row_format.format(*events2[-3].values())).rstrip()
            print (row_format.format(*events2[-2].values())).rstrip()
            print (row_format.format(*events2[-1].values())).rstrip()
            print 80 * '-' + '\n'

    if len(events) == 0:
        return 0

    # if read in event catalog, define variables new which determine
    # the name of the folder where results will be stored. (Other-
    # wise folder is named after defaults (10 & 5 days ago)). For
    # renaming use max and min from quakes of read catalog
    if (input_dics['read_catalog'] != 'N' and successful_read == 1) \
            or len(garbage) != 0:
        input_dics['max_date'] = str(max([e['datetime'] for e in events]))
        input_dics['min_date'] = str(min([e['datetime'] for e in events]))
        input_dics['max_mag'] = str(max([e['magnitude'] for e in events]))
        input_dics['min_mag'] = str(min([e['magnitude'] for e in events]))
    return events, catalog, events2, row_format, header, input_dics

# ##################### write_cat_logger ############################


def write_cat_logger(input_dics, eventpath, period, events, catalog,
                     events2, row_format, header):
    """
    Writing outputs out of get_Events function
    :param input_dics:
    :param eventpath:
    :param period:
    :param events:
    :param catalog:
    :param events2:
    :param row_format:
    :return:
    """
    if os.path.exists(eventpath):
        print '\n' + 50*'*'
        if raw_input('Directory for the requested period already exists:'
                     '\n%s\n\nOptions:\nN: Close the program and try '
                     'the updating mode.\nY: Remove the tree, continue the '
                     'program and re-download.\n' % eventpath).upper() == 'Y':
            print 50*'*'
            shutil.rmtree(eventpath)
            os.makedirs(eventpath)
        else:
            sys.exit('Exit the program ..')
    else:
        os.makedirs(eventpath)

    os.makedirs(os.path.join(eventpath, 'EVENTS-INFO'))
    input_logger(argus=sys.argv,
                 address=os.path.join(eventpath,
                                      'EVENTS-INFO',
                                      'logger_command.txt'),
                 inputs=input_dics)

    # output catalogue as ASCII
    Event_cat = \
        open(os.path.join(eventpath, 'EVENTS-INFO', 'catalog.txt'), 'a+')
    st_argus = 'Command line:\n-------------\n'
    for item in sys.argv:
        st_argus += item + ' '
    st_argus += '\n'
    Event_cat.writelines(str(period) + '\n')
    Event_cat.writelines(st_argus)
    Event_cat.writelines('-------------------------------------' + '\n')
    Event_cat.writelines('Information about the requested Events:' + '\n\n')
    Event_cat.writelines('Number of Events: %s\n' % len(events))
    Event_cat.writelines('min datetime: %s\n' % input_dics['min_date'])
    Event_cat.writelines('max datetime: %s\n' % input_dics['max_date'])
    Event_cat.writelines('min magnitude: %s\n' % input_dics['min_mag'])
    Event_cat.writelines('max magnitude: %s\n' % input_dics['max_mag'])
    Event_cat.writelines('min latitude: %s\n' % input_dics['evlatmin'])
    Event_cat.writelines('max latitude: %s\n' % input_dics['evlatmax'])
    Event_cat.writelines('min longitude: %s\n' % input_dics['evlonmin'])
    Event_cat.writelines('max longitude: %s\n' % input_dics['evlonmax'])
    Event_cat.writelines('min depth: %s\n' % input_dics['min_depth'])
    Event_cat.writelines('max depth: %s\n' % input_dics['max_depth'])
    Event_cat.writelines('\n\n-------------------------------------\n')
    Event_cat.close()

    for i in range(len(events)):
        Event_cat = open(os.path.join(eventpath, 'EVENTS-INFO',
                                      'catalog.txt'), 'a')
        Event_cat.writelines("Event No: %s\n" % (i+1))
        Event_cat.writelines("Author: %s\n" % events[i]['author'])
        Event_cat.writelines("Event-ID: %s\n" % events[i]['event_id'])
        Event_cat.writelines("Date Time: %s\n" % events[i]['datetime'])
        Event_cat.writelines("Magnitude: %s\n" % events[i]['magnitude'])
        Event_cat.writelines("Magnitude Type: %s\n"
                             % events[i]['magnitude_type'])
        Event_cat.writelines("Depth: %s\n" % events[i]['depth'])
        Event_cat.writelines("Latitude: %s\n" % events[i]['latitude'])
        Event_cat.writelines("Longitude: %s\n" % events[i]['longitude'])
        try:
            Event_cat.writelines("Flynn-Region: %s\n"
                                 % events[i]['flynn_region'])
        except KeyError:
            Event_cat.writelines("Flynn-Region: None\n")
        Event_cat.writelines('-------------------------------------' + '\n')
        Event_cat.close()
    Event_file = open(os.path.join(eventpath, 'EVENTS-INFO',
                                   'event_list_pickle'), 'a+')
    pickle.dump(events, Event_file)
    Event_file.close()

    # output catalogue as ASCII but in table form
    try:
        st_argus = 'Command line:\n-------------\n'
        for item in sys.argv:
            st_argus += item + ' '
        st_argus += '\n'
        Event_table = \
            open(os.path.join(eventpath, 'EVENTS-INFO',
                              'catalog_table.txt'), 'w')
        Event_table.writelines(st_argus)
        Event_table.writelines('\n' + row_format.format(*header))
        Event_table.writelines('-'*80 + '\n')
        for i in range(len(events2)):
            Event_table.writelines(
                (row_format.format(*events2[i].values())).rstrip() + '\n')
        Event_table.writelines('-'*80 + '\n')
    except Exception as err:
        print '\nCouldn\'t write catalog object to ASCII file as:\n>>:\t %s\n' \
              'Proceed without ..\n' % err

    # output catalogue as QUAKEML / JSON files
    try:
        catalog.write(
            os.path.join(eventpath, 'EVENTS-INFO', 'catalog.ml'),
            format="QUAKEML")
    except Exception as err:
        print '\nCouldn\'t write catalog object to QuakeML as:\n>>:\t %s\n' \
              'Proceed without ..\n' % err
    try:
        catalog.write(
            os.path.join(eventpath, 'EVENTS-INFO', 'catalog.json'),
            format="JSON")
    except Exception as err:
        print '\nCouldn\'t write catalog object to JSON as:\n>>:\t %s\n' \
              'Proceed without ..\n' % err

# ##################### neic_catalog ############################


def neic_catalog(t_start, t_end, min_latitude, max_latitude, min_longitude,
                 max_longitude, latitude, longitude, radius_min, radius_max,
                 d_min, d_max, mag_min, mag_max,
                 link_neic="http://earthquake.usgs.gov/earthquakes/search/"):
    """
    Function for downloading data from NEIC
    :param t_start:
    :param t_end:
    :param min_latitude:
    :param max_latitude:
    :param min_longitude:
    :param max_longitude:
    :param latitude:
    :param longitude:
    :param radius_min:
    :param radius_max:
    :param d_min:
    :param d_max:
    :param mag_min:
    :param mag_max:
    :param link_neic:
    :return:
    """
    try:
        import mechanize
    except Exception, e:
        sys.exit('ERROR:\nFor NEIC_USGS, "mechanize" should be installed: %s\n'
                 '\npip install mechanize\n' % e)

    tic = time.clock()

    dir_name = '%s_temp_xml_files' % int(UTCDateTime.now().timestamp)
    os.mkdir(dir_name)

    # Defines which website we want to look at
    br = mechanize.Browser()
    br.open(link_neic)

    # NEIC has two forms, but the relevant one for us is the first form!
    br.form1 = list(br.forms())[0]

    br.form1['minmagnitude'] = str(mag_min)
    br.form1['maxmagnitude'] = str(mag_max)
    br.form1['mindepth'] = str(d_min)
    br.form1['maxdepth'] = str(d_max)

    if None in [latitude, longitude, radius_min, radius_max]:
        if None in [min_latitude, max_latitude, min_longitude, max_longitude]:
            br.form1['minlongitude'] = '-180'
            br.form1['maxlongitude'] = '180'
            br.form1['minlatitude'] = '-90'
            br.form1['maxlatitude'] = '90'
        else:
            br.form1['maxlatitude'] = str(max_latitude)
            br.form1['minlongitude'] = str(min_longitude)
            br.form1['maxlongitude'] = str(max_longitude)
            br.form1['minlatitude'] = str(min_latitude)
    else:
        br.form1['latitude'] = str(latitude)
        br.form1['longitude'] = str(longitude)
        br.form1['minradiuskm'] = str(radius_min)
        br.form1['maxradiuskm'] = str(radius_max)

    # This function at this moment only provides these settings
    br.form1['format'] = ['quakeml']
    br.form1['includeallorigins'] = ['true']
    br.form1['includeallmagnitudes'] = ['true']
    br.form1['producttype'] = 'moment-tensor'

    m_date = UTCDateTime(t_start)
    M_date = UTCDateTime(t_end)
    dur_event = M_date - m_date
    interval = 30.*24.*60.*60.

    num_div = int(dur_event/interval)
    print 'Number of divisions: %s' % num_div
    # residual time is: (has not been used here)
    # t_res = t_cont - num_div*input_dics['interval']
    for i in range(1, num_div+1):
        print i,
        sys.stdout.flush()
        t_start_split = m_date + (i-1)*interval
        t_end_split = m_date + i*interval
        br.form1['starttime'] = str(t_start_split)
        br.form1['endtime'] = str(t_end_split)

        # Final output will be now porcessed
        request = br.form1.click()
        br.open(request)
        url = br.geturl()

        remotefile = ('%s' % url)
        page = urllib2.urlopen(remotefile)
        page_content = page.read()

        if 'quakeml' in page_content:
            with open(os.path.join(dir_name,
                                   'temp_neic_xml_%05i.xml' % i),
                      'w') as fid:
                fid.write(page_content)
            fid.close()
        else:
            continue

    final_time = m_date + num_div*interval
    if not M_date == final_time:
        t_start_split = m_date + num_div*interval
        t_end_split = M_date
        print '\nEnd time: %s\n' % t_end_split
        br.form1['starttime'] = str(t_start_split)
        br.form1['endtime'] = str(t_end_split)

        # Final output will be now porcessed
        request = br.form1.click()
        br.open(request)
        url = br.geturl()

        remotefile = ('%s' % url)
        page = urllib2.urlopen(remotefile)
        page_content = page.read()

        if 'quakeml' in page_content:
            with open(os.path.join(dir_name,
                                   'temp_neic_xml_%05i.xml' % (num_div+1)),
                      'w') as fid:
                fid.write(page_content)
            fid.close()

    xml_add = glob.glob(os.path.join(dir_name, 'temp_neic_xml_*.xml'))
    xml_add.sort()
    cat = Catalog()
    print 'Start assembling the xml files: %s...\n' % len(xml_add)
    counter = 1
    for x_add in xml_add:
        print counter,
        sys.stdout.flush()
        counter += 1
        try:
            cat.extend(readEvents(x_add, format='QuakeML'))
            os.remove(x_add)
        except Exception, e:
            print 'WARNING: %s' % e
            os.remove(x_add)

    os.rmdir(dir_name)
    toc = time.clock()
    print 'It took %s sec to retrieve the earthquakes form NEIC.' % (toc-tic)
    return cat

# ##################### gcmt_catalog ############################


def gcmt_catalog(t_start, t_end, min_latitude, max_latitude, min_longitude,
                 max_longitude, latitude, longitude, radius_min, radius_max,
                 d_min, d_max, mag_min, mag_max,
                 link_gcmt='http://www.ldeo.columbia.edu/~gcmt/projects/CMT/'
                           'catalog'):
    """
    Function for downloading data from GCMT
    :param t_start:
    :param t_end:
    :param min_latitude:
    :param max_latitude:
    :param min_longitude:
    :param max_longitude:
    :param latitude:
    :param longitude:
    :param radius_min:
    :param radius_max:
    :param d_min:
    :param d_max:
    :param mag_min:
    :param mag_max:
    :return:
    """
    # for the time record
    tic = datetime.now()

    if not os.path.exists('gcmt_catalog'):
        os.mkdir('gcmt_catalog')
        os.mkdir(os.path.join('gcmt_catalog', 'NEW_MONTHLY'))
        os.mkdir(os.path.join('gcmt_catalog', 'COMBO'))

    # creating a time list
    t_list = []
    delta_t = int(UTCDateTime(t_end)-UTCDateTime(t_start)+1)/86400

    yymm = []
    for i in range(delta_t + 1):
        t_list.append((UTCDateTime(t_start)+i*60*60*24).strftime('%Y/%m/%d'))
        yy_tmp, mm_tmp, dd_tmp = t_list[i].split('/')
        yymm.append('%s%s' % (yy_tmp, mm_tmp))
    yymmset = set(yymm)
    yymmls = list(yymmset)
    yymmls.sort()

    # starting to search for all events in the time window given by the user:
    cat = Catalog()
    yy_ret = []
    mm_ret = []
    remotefile_add = False
    try:
        for i in range(len(yymmls)):
            yy = yymmls[i][0:4]
            mm = yymmls[i][4:6]
            if int(yy) < 2006:
                month_year = ['jan', 'feb', 'mar', 'apr', 'may', 'june',
                              'july', 'aug', 'sept', 'oct', 'nov', 'dec']
            else:
                month_year = ['jan', 'feb', 'mar', 'apr', 'may', 'jun',
                              'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
            if int(yy) >= 2005:
                new_monthly = 'NEW_MONTHLY'
                file_to_open = os.path.join('gcmt_catalog', new_monthly,
                                            '%s%s.ndk'
                                            % (month_year[int(mm)-1], yy[-2:]))
                remotefile_add = '%s/%s/%s/%s%s.ndk' \
                                 % (link_gcmt, new_monthly, yy,
                                    month_year[int(mm)-1], yy[-2:])
            else:
                new_monthly = 'COMBO'
                if yy in yy_ret:
                    continue
                file_to_open = os.path.join('gcmt_catalog', new_monthly,
                                            '%s.qml' % yy)
            if not os.path.exists(file_to_open) and not new_monthly == 'COMBO':
                print 'Reading the data from GCMT webpage: %s' % yymmls[i]
                remotefile = urllib2.urlopen(remotefile_add)
                remotefile_read = remotefile.readlines()
                search_fio = open(file_to_open, 'w')
                search_fio.writelines(remotefile_read)
                search_fio.close()
            print 'Reading the data from local gcmt_catalog: %s' % yymmls[i]
            cat.extend(readEvents(file_to_open))
            yy_ret.append(yy)
            mm_ret.append(mm)
        print 'Done reading the data from GCMT webpage.'
    except Exception, e:
        print "ERROR: %s" % e

    toc = datetime.now()
    print 'It took %s to retrieve the earthquakes form GCMT.' % (toc-tic)

    filt1 = 'time >= %s' % t_start
    filt2 = 'time <= %s' % t_end
    cat = cat.filter(filt1, filt2)

    filt1 = 'magnitude >= %s' % mag_min
    filt2 = 'magnitude <= %s' % mag_max
    cat = cat.filter(filt1, filt2)

    filt1 = 'depth >= %s' % (float(d_min)*1000.)
    filt2 = 'depth <= %s' % (float(d_max)*1000.)
    cat = cat.filter(filt1, filt2)

    if None not in [min_latitude, max_latitude, min_longitude, max_longitude]:
        filt1 = 'latitude >= %s' % min_latitude
        filt2 = 'latitude <= %s' % max_latitude
        cat = cat.filter(filt1, filt2)

        filt1 = 'longitude >= %s' % min_longitude
        filt2 = 'longitude <= %s' % max_longitude
        cat = cat.filter(filt1, filt2)

    # final filtering for the remaining requests
    if None not in [latitude, longitude, radius_min, radius_max]:
        index_rm = []
        for i in range(len(cat)):
            e_lat = cat.events[i].preferred_origin().latitude or \
                    cat.events[i].origins[0].latitude
            e_lon = cat.events[i].preferred_origin().longitude or \
                    cat.events[i].origins[0].longitude
            dist = locations2degrees(latitude, longitude, e_lat, e_lon)
            if not radius_min <= dist <= radius_max:
                index_rm.append(i)
        index_rm.sort()
        index_rm.reverse()
        for i in range(len(index_rm)):
            del cat[index_rm[i]]

    return cat

# ##################### event_spaces ############################


def event_spaces(events, request):
    """
    calculate spaces used for table (shell ouput)
    :param events:
    :param request:
    :return:
    """
    header = ['None']
    events2 = copy.deepcopy(events)
    for i in range(len(events2)):
        for item_ev in ['t1', 't2', 'origin_id', 'magnitude_type',
                        'focal_mechanism', 'half_duration']:
            try:
                del events2[i][item_ev]
            except Exception, e:
                print 'WARNING: %s' % e
                pass
        try:
            events2[i]['datetime'] = str(events2[i]['datetime'])[:-8]
        except Exception, e:
            print 'WARNING: %s' % e
            pass
        try:
            if request == 'continuous':
                del events2[i]['flynn_region']
                header = ['#N', 'LAT', 'LON', 'DEP', 'DATETIME', 'MAG',
                          'AUTH', 'EV_ID']
                events2[i]['latitude'] = \
                    "{:>6}".format(int(events2[i]['latitude']))
                events2[i]['longitude'] = \
                    "{:>6}".format(int(events2[i]['longitude']))
                events2[i]['depth'] = \
                    "{:>6}".format(int(events2[i]['depth']))
            else:
                header = ['#N', 'LAT', 'LON', 'DEP', 'DATETIME', 'MAG',
                          'AUTH', 'EV_ID', 'FLY']
                events2[i]['latitude'] = \
                    "{:>8.3f}".format(float(events2[i]['latitude']))
                events2[i]['longitude'] = \
                    "{:>8.3f}".format(float(events2[i]['longitude']))
                events2[i]['depth'] = \
                    int(round(float(events2[i]['depth'])))
        except Exception, e:
            print 'WARNING: %s' % e
            pass

    try:
        k, spaces = [], []
        for i in range(len(events2)):
            k.append(events2[i].values())

        for j in range(len(k[0])):
            spaces.append(max([len(str(k[i][j])) for i in range(len(k))]))

        return spaces, events2, header
    except Exception, e:
        print 'WARNING: %s' % e
        pass

# ##################### delete_events ############################


def delete_events(events, catalog):
    """
    delete-event procedure
    :param events:
    :param catalog:
    :return:
    """
    garbage = []
    ev_num = raw_input(
        'Type the number of event you wish not to proceed with, '
        'then hit >Enter<.\nTo delete all events but number x, y .. '
        'type:  >!x[, y, z ..]<.\nTo proceed without making a selection, '
        'hit >Enter< now:\t')
    while re.search(r"\A\s*\d+\s*\Z|\A\s*!\s*\d+(\s*,\s*\d+)*\s*\Z", ev_num):
        if ev_num.strip()[0] == '!':
            keep = [int(i) for i in ev_num.strip().strip('!').split(',')]
            garbage = [i for i in range(1, len(events)+1) if i not in keep]
            break
        else:
            if int(ev_num) < 1 or int(ev_num) > len(events):
                break
            garbage.append(int(ev_num))
            ev_num = raw_input('Go on:\t')

    print '\nThe following events are not considered for ' \
          'further steps:\n%s\n' % garbage

    if len(garbage) != 0:
        garbage = list(set(garbage))
        garbage.sort()

        garbage.reverse()
        for ev_out in garbage:
            del events[ev_out-1]
            catalog.__delitem__(ev_out-1)
        if len(events) == 0:
            sys.exit('\nExit, seems like the catalog is '
                     'empty now. Try Again.')
    else:
        print

    return events, catalog, garbage

# ##################### sort_catalogue ############################


def sort_catalogue(cat):
    """
    Sort catalogue of retrieved events chronological.
    :param cat:
    :return:
    """
    k = [[event, event.origins[0].time] for event in cat]
    k.sort(key=lambda x: x[1], reverse=True)
    events = [event[0] for event in k]
    cat = Catalog(events=events)
    return cat

# ##################### quake_info ######################################


def quake_info(address, target):
    """
    Reads the info in quake file ("info" folder)
    :param address:
    :param target:
    :return:
    """
    events = []
    target_add = locate(address, target)
    for t_add in target_add:
        if not os.path.isfile(os.path.join(t_add, 'quake')):
            print '='*64
            print 'quake file could not be found'
            print 'Start Creating the quake file'
            print 'WARNING: it uses just one seismogram to ' \
                  're-create the quake file'
            print '='*64
            quake_create(address_info=t_add)
        quake_file_open = open(os.path.join(t_add, 'quake'), 'r')
        quake_file = quake_file_open.readlines()

        quake_read_tmp = []
        for q_file_l in quake_file:
            for q_l_item in q_file_l.split():
                try:
                    quake_read_tmp.append(float(q_l_item))
                except ValueError:
                    pass

        if len(quake_read_tmp) < 20:
            print '====================='
            print 'Modify the quake file'
            print '====================='
            quake_modify(quake_item=quake_read_tmp, address_info=t_add)

            quake_file_open = open(os.path.join(t_add, 'quake'), 'r')
            quake_file = quake_file_open.readlines()

            quake_read_tmp = []
            for q_file_l in quake_file:
                for q_l_item in q_file_l.split():
                    try:
                        quake_read_tmp.append(float(q_l_item))
                    except ValueError:
                        pass

        quake_d = {'year0': int(quake_read_tmp[0]),
                   'julday0': int(quake_read_tmp[1]),
                   'hour0': int(quake_read_tmp[2]),
                   'minute0': int(quake_read_tmp[3]),
                   'second0': int(quake_read_tmp[4]),
                   'micros0': int(quake_read_tmp[5]),
                   'lat': float(quake_read_tmp[6]),
                   'lon': float(quake_read_tmp[7]),
                   'dp': float(quake_read_tmp[8]),
                   'mag': float(quake_read_tmp[9]),
                   'year1': int(quake_read_tmp[10]),
                   'julday1': int(quake_read_tmp[11]),
                   'hour1': int(quake_read_tmp[14]),
                   'minute1': int(quake_read_tmp[15]),
                   'second1': int(quake_read_tmp[16]),
                   'micros1': int(quake_read_tmp[17]),
                   'year2': int(quake_read_tmp[18]),
                   'julday2': int(quake_read_tmp[19]),
                   'hour2': int(quake_read_tmp[22]),
                   'minute2': int(quake_read_tmp[23]),
                   'second2': int(quake_read_tmp[24]),
                   'micros2': int(quake_read_tmp[25])}

        quake_t0 = UTCDateTime(year=quake_d['year0'],
                               julday=quake_d['julday0'],
                               hour=quake_d['hour0'],
                               minute=quake_d['minute0'],
                               second=quake_d['second0'],
                               microsecond=quake_d['micros0'])
        quake_t1 = UTCDateTime(year=quake_d['year1'],
                               julday=quake_d['julday1'],
                               hour=quake_d['hour1'],
                               minute=quake_d['minute1'],
                               second=quake_d['second1'],
                               microsecond=quake_d['micros1'])
        quake_t2 = UTCDateTime(year=quake_d['year2'],
                               julday=quake_d['julday2'],
                               hour=quake_d['hour2'],
                               minute=quake_d['minute2'],
                               second=quake_d['second2'],
                               microsecond=quake_d['micros2'])

        events.append({'number': False,
                       'latitude': quake_d['lat'],
                       'longitude': quake_d['lon'],
                       'depth': quake_d['dp'],
                       'datetime': quake_t0,
                       'magnitude': quake_d['mag'],
                       'magnitude_type': 'NONE',
                       'author': 'NONE',
                       'event_id': quake_file[5].split('-')[0].lstrip(),
                       'origin_id': -12345.0,
                       'focal_mechanism': False,
                       'half_duration': False,
                       'flynn_region': 'NONE',
                       't1': quake_t1,
                       't2': quake_t2})

    address_event = []
    for t_add in target_add:
        address_event.append(os.path.dirname(t_add))

    return events, address_event

# ##################### quake_create ####################################


def quake_create(address_info):
    """
    if there is no quake file in the info folder it will be created
    based on the data available in the BH_RAW or BH file
    :param address_info:
    :return:
    """

    quake_file = open(os.path.join(address_info, 'quake'), 'w')
    address = os.path.normpath(os.path.join(address_info, '..'))

    if os.path.isdir(os.path.join(address, 'BH_RAW')):
        sta_address = os.path.join(address, 'BH_RAW')
    elif os.path.isdir(os.path.join(address, 'BH')):
        sta_address = os.path.join(address, 'BH')
    else:
        print '\nERROR: There is no reference (BH_RAW or BH) ' \
              'to create a quake file...'
        sta_address = None
    sta_stats = False
    sta_indx = 0
    try:
        ls_stas = glob.glob(os.path.join(sta_address, '*.*.*.*'))
        search_flag = True
        while search_flag:
            try:
                sta = read(ls_stas[sta_indx])[sta_indx]
                sta_stats = sta.stats
                search_flag = False
            except Exception, e:
                search_flag = True
                sta_indx += 1
                print 'EXCEPTION: %s' % e
                pass
        print '\nCreate the quake file based on: \n%s' % ls_stas[sta_indx]
        quake_file.writelines(repr(sta_stats.starttime.year).rjust(15) +
                              repr(sta_stats.starttime.julday).rjust(15) +
                              '\n')
        quake_file.writelines(repr(sta_stats.starttime.hour).rjust(15) +
                              repr(sta_stats.starttime.minute).rjust(15) +
                              repr(sta_stats.starttime.second).rjust(15) +
                              repr(sta_stats.starttime.microsecond).rjust(15)
                              + '\n')
        quake_file.writelines(' '*(15 - len('%.5f' % sta_stats.sac.evla)) +
                              '%.5f' % sta_stats.sac.evla +
                              ' '*(15 - len('%.5f' % sta_stats.sac.evlo)) +
                              '%.5f' % sta_stats.sac.evlo + '\n')
        quake_file.writelines(' '*(15 - len('%.5f' % abs(sta_stats.sac.evdp)))
                              + '%.5f' % abs(sta_stats.sac.evdp) + '\n')
        quake_file.writelines(' '*(15 - len('%.5f' % abs(sta_stats.sac.mag))) +
                              '%.5f' % abs(sta_stats.sac.mag) + '\n')
        quake_file.writelines(' '*(15 - len(address.split('/')[-1])) +
                              address.split('/')[-1] + '-' + '\n')

        quake_file.writelines(repr(sta_stats.starttime.year).rjust(15) +
                              repr(sta_stats.starttime.julday).rjust(15) +
                              repr(sta_stats.starttime.month).rjust(15) +
                              repr(sta_stats.starttime.day).rjust(15) + '\n')
        quake_file.writelines(repr(sta_stats.starttime.hour).rjust(15) +
                              repr(sta_stats.starttime.minute).rjust(15) +
                              repr(sta_stats.starttime.second).rjust(15) +
                              repr(sta_stats.starttime.microsecond).rjust(15)
                              + '\n')

        sta_stats_endtime = \
            sta_stats.starttime + (sta_stats.npts-1)/sta_stats.sampling_rate

        quake_file.writelines(
            '{0}{1}{2}{3}\n'.format(repr(sta_stats_endtime.year).rjust(15),
                                    repr(sta_stats_endtime.julday).rjust(15),
                                    repr(sta_stats_endtime.month).rjust(15),
                                    repr(sta_stats_endtime.day).rjust(15)))
        quake_file.writelines(
            '{0}{1}{2}{3}\n'.format(repr(sta_stats_endtime.hour).rjust(15),
                                    repr(sta_stats_endtime.minute).rjust(15),
                                    repr(sta_stats_endtime.second).rjust(15),
                                    repr(sta_stats_endtime.microsecond).
                                    rjust(15)))

    except Exception as e:
        print '\n================='
        print 'WARNING: Can not read all the required information ' \
              'from the header'
        print 'Following parameters are presumed:'
        print 'evla=0, evlo=0, dp=-12345.0, mag=-12345.0'
        print 'Exception: %s' % e
        print '=================\n'
        quake_file.writelines(repr(sta_stats.starttime.year).rjust(15) +
                              repr(sta_stats.starttime.julday).rjust(15) +
                              '\n')
        quake_file.writelines(repr(sta_stats.starttime.hour).rjust(15) +
                              repr(sta_stats.starttime.minute).rjust(15) +
                              repr(sta_stats.starttime.second).rjust(15) +
                              repr(sta_stats.starttime.microsecond).rjust(15)
                              + '\n')
        quake_file.writelines(' '*(15 - len('%.5f' % 0.0)) + '%.5f' % 0.0 +
                              ' '*(15 - len('%.5f' % 0.0)) + '%.5f' % 0.0
                              + '\n')
        quake_file.writelines(' '*(15 - len('%.5f' % abs(-12345.0))) +
                              '%.5f' % abs(-12345.0) + '\n')
        quake_file.writelines(' '*(15 - len('%.5f' % abs(-12345.0))) +
                              '%.5f' % abs(-12345.0) + '\n')
        quake_file.writelines(' '*(15 - len(address.split('/')[-1])) +
                              address.split('/')[-1] + '-' + '\n')

        quake_file.writelines(repr(sta_stats.starttime.year).rjust(15) +
                              repr(sta_stats.starttime.julday).rjust(15) +
                              repr(sta_stats.starttime.month).rjust(15) +
                              repr(sta_stats.starttime.day).rjust(15) + '\n')
        quake_file.writelines(repr(sta_stats.starttime.hour).rjust(15) +
                              repr(sta_stats.starttime.minute).rjust(15) +
                              repr(sta_stats.starttime.second).rjust(15) +
                              repr(sta_stats.starttime.microsecond).rjust(15)
                              + '\n')

        sta_stats_endtime = \
            sta_stats.starttime + (sta_stats.npts-1)/sta_stats.sampling_rate

        quake_file.writelines(repr(sta_stats_endtime.year).rjust(15) +
                              repr(sta_stats_endtime.julday).rjust(15) +
                              repr(sta_stats_endtime.month).rjust(15) +
                              repr(sta_stats_endtime.day).rjust(15) + '\n')
        quake_file.writelines(repr(sta_stats_endtime.hour).rjust(15) +
                              repr(sta_stats_endtime.minute).rjust(15) +
                              repr(sta_stats_endtime.second).rjust(15) +
                              repr(sta_stats_endtime.microsecond).rjust(15) +
                              '\n')
    quake_file.close()

# ##################### quake_modify ####################################


def quake_modify(quake_item, address_info):
    """
    if the quake file does not contain all the required parameters
    it will be modified based on the available data in BH_RAW or BH directories
    :param quake_item:
    :param address_info:
    :return:
    """

    quake_file_new = open(os.path.join(address_info, 'quake'), 'w')
    address = os.path.normpath(os.path.join(address_info, '..'))

    if os.path.isdir(os.path.join(address, 'BH_RAW')):
        sta_address = os.path.join(address, 'BH_RAW')
    elif os.path.isdir(os.path.join(address, 'BH')):
        sta_address = os.path.join(address, 'BH')
    else:
        print '\nERROR: There is no reference (BH_RAW or BH) ' \
              'to modify the quake file...'
        sta_address = None

    ls_stas = glob.glob(os.path.join(sta_address, '*.*.*.*'))

    sta = read(ls_stas[0])[0]
    sta_stats = sta.stats
    print '\nCreate the quake file based on: \n%s' % ls_stas[0]

    try:
        quake_file_new.writelines(repr(int(quake_item[0])).rjust(15) +
                                  repr(int(quake_item[1])).rjust(15) + '\n')
        quake_file_new.writelines(repr(int(quake_item[2])).rjust(15) +
                                  repr(int(quake_item[3])).rjust(15) +
                                  repr(int(quake_item[4])).rjust(15) +
                                  repr(int(quake_item[5])).rjust(15) + '\n')
        quake_file_new.writelines(' '*(15 - len('%.5f' % quake_item[6])) +
                                  '%.5f' % quake_item[6] +
                                  ' '*(15 - len('%.5f' % quake_item[7])) +
                                  '%.5f' % quake_item[7] + '\n')
        quake_file_new.writelines(' '*(15 - len('%.5f' % abs(quake_item[8]))) +
                                  '%.5f' % abs(quake_item[8]) + '\n')
        quake_file_new.writelines(' '*(15 - len('%.5f'
                                                % abs(sta_stats.sac.mag))) +
                                  '%.5f' % abs(sta_stats.sac.mag) + '\n')
        quake_file_new.writelines(' '*(15 - len(address.split('/')[-1])) +
                                  address.split('/')[-1] + '-' + '\n')

        quake_file_new.writelines(repr(sta_stats.starttime.year).rjust(15) +
                                  repr(sta_stats.starttime.julday).rjust(15) +
                                  repr(sta_stats.starttime.month).rjust(15) +
                                  repr(sta_stats.starttime.day).rjust(15) +
                                  '\n')
        quake_file_new.writelines(repr(sta_stats.starttime.hour).rjust(15) +
                                  repr(sta_stats.starttime.minute).rjust(15) +
                                  repr(sta_stats.starttime.second).rjust(15) +
                                  repr(sta_stats.starttime.microsecond).
                                  rjust(15) + '\n')

        sta_stats_endtime = \
            sta_stats.starttime + (sta_stats.npts-1)/sta_stats.sampling_rate

        quake_file_new.writelines(repr(sta_stats_endtime.year).rjust(15) +
                                  repr(sta_stats_endtime.julday).rjust(15) +
                                  repr(sta_stats_endtime.month).rjust(15) +
                                  repr(sta_stats_endtime.day).rjust(15) + '\n')
        quake_file_new.writelines(repr(sta_stats_endtime.hour).rjust(15) +
                                  repr(sta_stats_endtime.minute).rjust(15) +
                                  repr(sta_stats_endtime.second).rjust(15) +
                                  repr(sta_stats_endtime.microsecond).rjust(15)
                                  + '\n')
    except Exception as e:
        print '\n================='
        print 'WARNING: Can not read all the required information ' \
              'from the header'
        print 'Following parameters are presumed:'
        print 'evla=0, evlo=0, dp=-12345.0, mag=-12345.0'
        print 'Exception: %s' % e
        print '=================\n'
        quake_file_new.writelines(repr(int(quake_item[0])).rjust(15) +
                                  repr(int(quake_item[1])).rjust(15) + '\n')
        quake_file_new.writelines(repr(int(quake_item[2])).rjust(15) +
                                  repr(int(quake_item[3])).rjust(15) +
                                  repr(int(quake_item[4])).rjust(15) +
                                  repr(int(quake_item[5])).rjust(15) + '\n')
        quake_file_new.writelines(' '*(15 - len('%.5f' % quake_item[6])) +
                                  '%.5f' % quake_item[6] +
                                  ' '*(15 - len('%.5f' % quake_item[7])) +
                                  '%.5f' % quake_item[7] + '\n')
        quake_file_new.writelines(' '*(15 - len('%.5f' % abs(quake_item[8])))
                                  + '%.5f' % abs(quake_item[8]) + '\n')
        quake_file_new.writelines(' '*(15 - len('%.5f' % abs(-12345.0)))
                                  + '%.5f' % abs(-12345.0) + '\n')
        quake_file_new.writelines(' '*(15 - len(address.split('/')[-1])) +
                                  address.split('/')[-1] + '-' + '\n')

        quake_file_new.writelines(repr(sta_stats.starttime.year).rjust(15) +
                                  repr(sta_stats.starttime.julday).rjust(15) +
                                  repr(sta_stats.starttime.month).rjust(15) +
                                  repr(sta_stats.starttime.day).rjust(15) +
                                  '\n')
        quake_file_new.writelines(repr(sta_stats.starttime.hour).rjust(15) +
                                  repr(sta_stats.starttime.minute).rjust(15) +
                                  repr(sta_stats.starttime.second).rjust(15) +
                                  repr(sta_stats.starttime.microsecond).
                                  rjust(15) + '\n')

        sta_stats_endtime = \
            sta_stats.starttime + (sta_stats.npts-1)/sta_stats.sampling_rate

        quake_file_new.writelines(repr(sta_stats_endtime.year).rjust(15) +
                                  repr(sta_stats_endtime.julday).rjust(15) +
                                  repr(sta_stats_endtime.month).rjust(15) +
                                  repr(sta_stats_endtime.day).rjust(15) + '\n')
        quake_file_new.writelines(repr(sta_stats_endtime.hour).rjust(15) +
                                  repr(sta_stats_endtime.minute).rjust(15) +
                                  repr(sta_stats_endtime.second).rjust(15) +
                                  repr(sta_stats_endtime.microsecond).rjust(15)
                                  + '\n')
    quake_file_new.close()

# ##################### create_folders_files ############################


def create_folders_files(events, eventpath, input_dics):
    """
    Create required folders and files in the event folder(s)
    :param events:
    :param eventpath:
    :param input_dics:
    :return:
    """
    for i in range(len(events)):
        try:
            os.makedirs(os.path.join(eventpath, events[i]['event_id'],
                                     'BH_RAW'))
            os.makedirs(os.path.join(eventpath, events[i]['event_id'],
                                     'Resp'))
            os.makedirs(os.path.join(eventpath, events[i]['event_id'],
                                     'info'))

            inp_file = open(os.path.join(eventpath, events[i]['event_id'],
                                         'info', 'input_dics.pkl'), 'w')
            pickle.dump(input_dics, inp_file)
            inp_file.close()
            report = open(os.path.join(eventpath, events[i]['event_id'],
                                       'info', 'report_st'), 'a+')
            report.close()
            exception_file = open(os.path.join(eventpath,
                                               events[i]['event_id'],
                                               'info', 'exception'), 'a+')
            exception_file.writelines('\n' + events[i]['event_id'] + '\n')
            exception_file.close()
            syn_file = open(os.path.join(eventpath, events[i]['event_id'],
                                         'info', 'station_event'), 'a+')
            syn_file.close()
        except Exception as e:
            print 'ERROR: %s' % e
            pass

    for i in range(len(events)):
        quake_file = open(os.path.join(eventpath, events[i]['event_id'],
                                       'info', 'quake'), 'a+')

        quake_file.writelines(repr(events[i]['datetime'].year).rjust(15) +
                              repr(events[i]['datetime'].julday).rjust(15)
                              + '\n')
        quake_file.writelines(repr(events[i]['datetime'].hour).rjust(15) +
                              repr(events[i]['datetime'].minute).rjust(15) +
                              repr(events[i]['datetime'].second).rjust(15) +
                              repr(events[i]['datetime'].microsecond).rjust(15)
                              + '\n')

        quake_file.writelines(' '*(15 - len('%.5f' % events[i]['latitude'])) +
                              '%.5f' % events[i]['latitude'] +
                              ' '*(15 - len('%.5f' % events[i]['longitude'])) +
                              '%.5f\n' % events[i]['longitude'])
        quake_file.writelines(' '*(15 - len('%.5f' % abs(events[i]['depth'])))
                              + '%.5f\n' % abs(events[i]['depth']))
        quake_file.writelines(' '*(15 -
                                   len('%.5f' % abs(events[i]['magnitude'])))
                              + '%.5f\n'
                              % abs(events[i]['magnitude']))
        quake_file.writelines(' '*(15 - len(events[i]['event_id'])) +
                              events[i]['event_id'] + '-' + '\n')

        quake_file.writelines(repr(events[i]['t1'].year).rjust(15) +
                              repr(events[i]['t1'].julday).rjust(15) +
                              repr(events[i]['t1'].month).rjust(15) +
                              repr(events[i]['t1'].day).rjust(15) + '\n')
        quake_file.writelines(repr(events[i]['t1'].hour).rjust(15) +
                              repr(events[i]['t1'].minute).rjust(15) +
                              repr(events[i]['t1'].second).rjust(15) +
                              repr(events[i]['t1'].microsecond).rjust(15) +
                              '\n')

        quake_file.writelines(repr(events[i]['t2'].year).rjust(15) +
                              repr(events[i]['t2'].julday).rjust(15) +
                              repr(events[i]['t2'].month).rjust(15) +
                              repr(events[i]['t2'].day).rjust(15) + '\n')
        quake_file.writelines(repr(events[i]['t2'].hour).rjust(15) +
                              repr(events[i]['t2'].minute).rjust(15) +
                              repr(events[i]['t2'].second).rjust(15) +
                              repr(events[i]['t2'].microsecond).rjust(15) +
                              '\n')

# ##################### create_tar_file #######################################


def create_tar_file(input_dics, address):
    """
    create a tar file out of a given directory
    :param input_dics:
    :param address:
    :return:
    """
    print '\n**************************'
    print 'Start creating tar file(s)'
    print '**************************'
    events, address_events = quake_info(address, 'info')
    for i in range(len(events)):
        # ---------Creating Tar files (Waveform files)
        if input_dics['zip_w'] == 'Y':
            print 'Compressing Raw files...'
            path = os.path.join(address_events[i], 'BH_RAW')
            tar_file = os.path.join(path, 'BH_RAW.tar')
            files = '*.*.*.*'
            compress_gzip(path=path, tar_file=tar_file, files=files)

        # ---------Creating Tar files (Response files)
        if input_dics['zip_r'] == 'Y':
            print 'Compressing Resp files...'
            path = os.path.join(address_events[i], 'Resp')
            tar_file = os.path.join(path, 'Resp.tar')
            files = '*.*.*.*'
            compress_gzip(path=path, tar_file=tar_file, files=files)

# ##################### compress_gzip ###################################


def compress_gzip(path, tar_file, files):
    """
    Compressing files and creating a tar file
    :param path:
    :param tar_file:
    :param files:
    :return:
    """
    tar = tarfile.open(tar_file, "w:gz")
    os.chdir(path)

    for infile in glob.glob(os.path.join(path, files)):
        print '.',
        tar.add(os.path.basename(infile))
        os.remove(infile)
    tar.close()

# ##################### mag_halfduration ###################################


def mag_halfduration(mag, type_curve=1):
    """
    Calculate the half_duration out of magnitude
    type_curve can be 1, 2, 3:
    1: 2005-2014
    2: 1976-1990
    3: 1976-2014
    :param mag:
    :param type_curve:
    :return:
    """
    if type_curve == 1:
        half_duration = 0.00272*np.exp(1.134*mag)
    elif type_curve == 2:
        half_duration = 0.00804*np.exp(1.025*mag)
    elif type_curve == 3:
        half_duration = 0.00392*np.exp(1.101*mag)
    else:
        sys.exit('%s Type for magnitude to half_duration conversion is not '
                 'implemented' % type_curve)
    return half_duration



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  arclink_handler.py
#   Purpose:   handling ArcLink in obspyDMT
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
from datetime import datetime
import multiprocessing
try:
    from obspy.clients.arclink import Client as Client_arclink
except Exception, e:
    from obspy.arclink import Client as Client_arclink
from obspy.core import UTCDateTime
import os
import pickle
import time

from event_handler import quake_info, create_folders_files
from format_converter import writesac_all
from utility_codes import read_list_stas, calculate_time_phase, getFolderSize

# ##################### Arclink_network #################################


def ARC_network(input_dics, events):
    """
    Returns information about what time series data is available
    at the ArcLink nodes for all requested events
    """
    print '\n***********************************************************'
    print 'ArcLink -- Download waveforms, response files and meta-data'
    print '***********************************************************'
    period = '{0:s}_{1:s}_{2:s}_{3:s}'.format(
        input_dics['min_date'].split('T')[0],
        input_dics['max_date'].split('T')[0],
        str(input_dics['min_mag']),
        str(input_dics['max_mag']))
    eventpath = os.path.join(input_dics['datapath'], period)

    if input_dics['FDSN'] != 'Y':
        print 'Initializing folders and files...',
        create_folders_files(events, eventpath, input_dics)
        print 'DONE'

    for i in range(len(events)):
        t_arc_1 = datetime.now()
        target_path = os.path.join(eventpath, events[i]['event_id'])

        if not input_dics['list_stas']:
            Stas_arc = ARC_available(input_dics, events[i],
                                     target_path, event_number=i)
        else:
            Stas_arc = read_list_stas(input_dics['list_stas'],
                                      normal_mode_syn='N',
                                      specfem3D='N')

        print '\nArcLink-Availability for event: %s/%s  ---> DONE' \
              % (i+1, len(events))
        print 'Time for checking the availability: %s' \
              % (datetime.now() - t_arc_1)

        if Stas_arc != [[]]:
            ARC_waveform(input_dics, events, Stas_arc, i, req_type='save')
        else:
            print 'No available station in ArcLink for your request and ' \
                  'for event %s!' % str(i+1)
            continue

# ##################### ARC_available ###################################


def ARC_available(input_dics, event, target_path, event_number):
    """
    Check the availability of ArcLink stations
    """
    print "Check the availability of ArcLink stations"
    client_arclink = Client_arclink(user='test@obspy.org',
                                    timeout=input_dics['arc_avai_timeout'])
    Sta_arc = []
    try:
        inventories = client_arclink.getInventory(
            network=input_dics['net'],
            station=input_dics['sta'],
            location=input_dics['loc'],
            channel=input_dics['cha'],
            starttime=UTCDateTime(event['t1']),
            endtime=UTCDateTime(event['t2']),
            min_latitude=input_dics['mlat_rbb'],
            max_latitude=input_dics['Mlat_rbb'],
            min_longitude=input_dics['mlon_rbb'],
            max_longitude=input_dics['Mlon_rbb'])

        for inv_key in inventories.keys():
            netsta = inv_key.split('.')
            if len(netsta) == 4:
                sta = '%s.%s' % (netsta[0], netsta[1])
                if not inventories[sta]['depth']:
                    inventories[sta]['depth'] = 0.0
                Sta_arc.append([netsta[0], netsta[1], netsta[2], netsta[3],
                                inventories[sta]['latitude'],
                                inventories[sta]['longitude'],
                                inventories[sta]['elevation'],
                                inventories[sta]['depth']])

    except Exception as e:
        exc_file = open(os.path.join(target_path, 'info', 'exception'), 'a+')
        ee = 'arclink -- Event: %s --- %s\n' % (str(event_number+1), e)
        exc_file.writelines(ee)
        exc_file.close()
        print 'ERROR: %s' % ee

    if len(Sta_arc) == 0:
        Sta_arc.append([])
    Sta_arc.sort()
    return Sta_arc

# ##################### Arclink_waveform ############################


def ARC_waveform(input_dics, events, Sta_req, i, req_type):
    """
    Gets Waveforms, Response files and meta-data from ArcLink
    """
    t_wave_1 = datetime.now()

    add_event = []
    if req_type == 'save':
        period = '{0:s}_{1:s}_{2:s}_{3:s}'.\
            format(input_dics['min_date'].split('T')[0],
                   input_dics['max_date'].split('T')[0],
                   str(input_dics['min_mag']),
                   str(input_dics['max_mag']))
        eventpath = os.path.join(input_dics['datapath'], period)
        for k in range(len(events)):
            add_event.append(os.path.join(eventpath, events[k]['event_id']))
            if not os.path.isfile(os.path.join(add_event[k],
                                               'info', 'event.pkl')):
                events_fio = open(os.path.join(add_event[k],
                                               'info', 'event.pkl'), 'w')
                pickle.dump(events[k], events_fio)
                events_fio.close()
    elif req_type == 'update':
        events, add_event = \
            quake_info(input_dics['arc_update'], target='info')

    if input_dics['test'] == 'Y':
        len_req_arc = input_dics['test_num']
    else:
        len_req_arc = len(Sta_req)

    ARC_serial_parallel(i, events, add_event, Sta_req, input_dics,
                        len_req_arc)
    if input_dics['SAC'] == 'Y':
        print '\nConverting the MSEED files to SAC...',
        writesac_all(i=i, address_events=add_event)
        print 'DONE'

    try:
        len_sta_ev_open = open(os.path.join(add_event[i], 'info',
                                            'station_event'), 'r')
        len_sta_ev = len(len_sta_ev_open.readlines())
    except IOError:
        len_sta_ev = 'Can not open station_event file: %s' \
                     % (os.path.join(add_event[i], 'info', 'station_event'))

    ARC_reporter(i, add_event, events, input_dics, Sta_req, len_sta_ev,
                 req_type, t_wave_1)

# ##################### ARC_serial_parallel ############################


def ARC_serial_parallel(i, events, add_event, Sta_req, input_dics,
                        len_req_arc):
    """
    ArcLink serial/parallel request
    """
    dic = {}
    print '\nArcLink-Event: %s/%s' % (i+1, len(events))

    client_arclink = Client_arclink(user='test@obspy.org',
                                    timeout=input_dics['arc_wave_timeout'])

    if input_dics['req_parallel'] == 'Y':
        print "Parallel request with %s processes.\n" % input_dics['req_np']
        parallel_len_req_arc = range(len_req_arc)
        len_par_grp = [parallel_len_req_arc[n:n+input_dics['req_np']] for n in
                       range(0, len(parallel_len_req_arc),
                             input_dics['req_np'])]
        par_jobs = []
        for j in range(len_req_arc):
            p = multiprocessing.Process(target=ARC_download_core,
                                        args=(i, j, dic, len(events),
                                              events, add_event, Sta_req,
                                              input_dics, client_arclink))
            par_jobs.append(p)
        for l in range(len(len_par_grp)):
            for ll in len_par_grp[l]:
                par_jobs[ll].start()
                time.sleep(0.01)
            for ll in len_par_grp[l]:
                while par_jobs[ll].is_alive():
                    time.sleep(0.01)
    else:
        for j in range(len_req_arc):
            ARC_download_core(i=i, j=j, dic=dic, len_events=len(events),
                              events=events, add_event=add_event,
                              Sta_req=Sta_req, input_dics=input_dics,
                              client_arclink=client_arclink)

# ##################### ARC_download_core ###############################


def ARC_download_core(i, j, dic, len_events, events, add_event,
                      Sta_req, input_dics, client_arclink):
    """
    Downloading waveforms, response files and metadata
    This program should be normally called by some higher-level functions
    """

    dummy = 'Initializing'
    info_req = 'None'
    t11 = datetime.now()
    try:
        info_req = '[%s/%s-%s/%s-%s] ' % (i+1, len_events, j+1,
                                          len(Sta_req), input_dics['cha'])

        if Sta_req[j][2] == '--' or Sta_req[j][2] == '  ':
                Sta_req[j][2] = ''

        if input_dics['cut_time_phase']:
            t_start, t_end = calculate_time_phase(events[i], Sta_req[j])
        else:
            t_start = events[i]['t1']
            t_end = events[i]['t2']

        if input_dics['waveform'] == 'Y':
            dummy = 'Waveform'
            client_arclink.saveWaveform(os.path.join(add_event[i],
                                                     'BH_RAW',
                                                     '%s.%s.%s.%s'
                                                     % (Sta_req[j][0],
                                                        Sta_req[j][1],
                                                        Sta_req[j][2],
                                                        Sta_req[j][3])),
                                        Sta_req[j][0], Sta_req[j][1],
                                        Sta_req[j][2], Sta_req[j][3],
                                        t_start, t_end)

            print '%ssaving waveform for: %s.%s.%s.%s  ---> DONE' \
                  % (info_req, Sta_req[j][0], Sta_req[j][1],
                     Sta_req[j][2], Sta_req[j][3])

        if input_dics['response'] == 'Y':
            dummy = 'Response'
            client_arclink.saveResponse(os.path.join(add_event[i], 'Resp',
                                                     'DATALESS.%s.%s.%s.%s'
                                                     % (Sta_req[j][0],
                                                        Sta_req[j][1],
                                                        Sta_req[j][2],
                                                        Sta_req[j][3])),
                                        Sta_req[j][0], Sta_req[j][1],
                                        Sta_req[j][2], Sta_req[j][3],
                                        t_start, t_end)

            print "%ssaving Response for: %s.%s.%s.%s  ---> DONE" \
                  % (info_req, Sta_req[j][0], Sta_req[j][1],
                     Sta_req[j][2], Sta_req[j][3])

        if input_dics['paz'] == 'Y':
            dummy = 'PAZ'
            paz_arc = client_arclink.getPAZ(Sta_req[j][0], Sta_req[j][1],
                                            Sta_req[j][2], Sta_req[j][3],
                                            time=t_start)
            paz_file = open(os.path.join(add_event[i], 'Resp',
                                         'PAZ.%s.%s.%s.%s.paz'
                                         % (Sta_req[j][0], Sta_req[j][1],
                                            Sta_req[j][2], Sta_req[j][3])),
                            'w')
            pickle.dump(paz_arc, paz_file)
            paz_file.close()

            print "%ssaving PAZ for     : %s.%s.%s.%s  ---> DONE" \
                  % (info_req, Sta_req[j][0], Sta_req[j][1],
                     Sta_req[j][2], Sta_req[j][3])

        dummy = 'Meta-data'
        dic[j] = {'info': '%s.%s.%s.%s' % (Sta_req[j][0], Sta_req[j][1],
                                           Sta_req[j][2], Sta_req[j][3]),
                  'net': Sta_req[j][0],
                  'sta': Sta_req[j][1],
                  'latitude': Sta_req[j][4],
                  'longitude': Sta_req[j][5],
                  'loc': Sta_req[j][2],
                  'cha': Sta_req[j][3],
                  'elevation': Sta_req[j][6],
                  'depth': Sta_req[j][7]}
        Syn_file = open(os.path.join(add_event[i], 'info',
                                     'station_event'), 'a')
        syn = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,arc,\n' \
              % (dic[j]['net'], dic[j]['sta'],
                 dic[j]['loc'], dic[j]['cha'],
                 dic[j]['latitude'], dic[j]['longitude'],
                 float(dic[j]['elevation']),
                 float(dic[j]['depth']),
                 events[i]['event_id'], events[i]['latitude'],
                 events[i]['longitude'], events[i]['depth'],
                 events[i]['magnitude'])
        Syn_file.writelines(syn)
        Syn_file.close()

        print "%ssaving Metadata for: %s.%s.%s.%s  ---> DONE" \
              % (info_req, Sta_req[j][0], Sta_req[j][1],
                 Sta_req[j][2], Sta_req[j][3])

        t22 = datetime.now()
        if input_dics['time_arc'] == 'Y':
            time_arc = t22 - t11
            time_file = open(os.path.join(add_event[i], 'info',
                                          'time_arc'), 'a')
            size = getFolderSize(os.path.join(add_event[i]))
            ti = '%s,%s,%s,%s,%s,%s,%s,+,\n' % (Sta_req[j][0],
                                                Sta_req[j][1],
                                                Sta_req[j][2],
                                                Sta_req[j][3],
                                                time_arc.seconds,
                                                time_arc.microseconds,
                                                size/(1024.**2))
            time_file.writelines(ti)
            time_file.close()
    except Exception as e:
        t22 = datetime.now()
        if input_dics['time_arc'] == 'Y':
            time_arc = t22 - t11
            time_file = open(os.path.join(add_event[i], 'info',
                                          'time_arc'), 'a')
            size = getFolderSize(os.path.join(add_event[i]))
            ti = '%s,%s,%s,%s,%s,%s,%s,-,\n' % (Sta_req[j][0],
                                                Sta_req[j][1],
                                                Sta_req[j][2],
                                                Sta_req[j][3],
                                                time_arc.seconds,
                                                time_arc.microseconds,
                                                size/(1024.**2))
            time_file.writelines(ti)
            time_file.close()

        if len(Sta_req[j]) != 0:
            print '%s%s---%s.%s.%s.%s' % (info_req, dummy,
                                          Sta_req[j][0],
                                          Sta_req[j][1],
                                          Sta_req[j][2],
                                          Sta_req[j][3])
            ee = 'arc -- %s---%s-%s---%s.%s.%s.%s---%s\n' \
                 % (dummy, i+1, j+1,
                    Sta_req[j][0], Sta_req[j][1], Sta_req[j][2],
                    Sta_req[j][3], e)
        else:
            ee = 'There is no available station for this event.'
        Exception_file = open(os.path.join(add_event[i],
                                           'info', 'exception'), 'a')
        Exception_file.writelines(ee)
        Exception_file.close()
        print 'ERROR: %s' % ee

# ##################### ARC_reporter ###############################


def ARC_reporter(i, add_event, events, input_dics, Sta_req, len_sta_ev,
                 req_type, t_wave_1):
    """
    Writing reports for the request
    :param i:
    :param add_event:
    :param events:
    :param input_dics:
    :param Sta_req:
    :param len_sta_ev:
    :param req_type:
    :param t_wave_1:
    :return:
    """
    report = open(os.path.join(add_event[i], 'info', 'report_st'), 'a')
    eventsID = events[i]['event_id']
    report.writelines('<><><><><><><><><><><><><><><><><>\n')
    report.writelines(eventsID + '\n')
    report.writelines('---------------ARC---------------\n')
    report.writelines('---------------%s---------------\n' % input_dics['cha'])
    rep = 'ARC-Available stations for channel %s and for event-%s: %s\n' \
          % (input_dics['cha'], i, len(Sta_req))
    report.writelines(rep)
    rep = 'ARC-%s stations for channel %s and for event-%s: %s\n' \
          % (req_type, input_dics['cha'], i, len_sta_ev)
    report.writelines(rep)
    report.writelines('----------------------------------\n')

    t_wave = datetime.now() - t_wave_1

    rep = 'Time for %sing Waveforms from ArcLink: %s\n' % (req_type, t_wave)
    report.writelines(rep)
    report.writelines('----------------------------------\n')
    report.close()

    if input_dics['req_parallel'] == 'Y':
        report_parallel_open = open(os.path.join(add_event[i], 'info',
                                                 'report_parallel'), 'a')
        report_parallel_open.writelines('---------------ARC---------------\n')
        report_parallel_open.writelines('Request\n')
        report_parallel_open.writelines('Number of Nodes: %s\n'
                                        % input_dics['req_np'])

        size = getFolderSize(os.path.join(add_event[i]))
        ti = '%s,%s,%s,+,\n' % (t_wave.seconds,
                                t_wave.microseconds,
                                size/(1024.**2))

        report_parallel_open.writelines('Total Time     : %s\n' % t_wave)
        report_parallel_open.writelines(ti)
        report_parallel_open.close()

    print "\n------------------------"
    print 'ArcLink for event-%s is Done' % (i+1)
    print 'Total Time: %s' % t_wave
    print "------------------------"

# -------------------------------- TRASH
# par_jobs = []
# for j in range(len_req_arc):
#     p = multiprocessing.Process(target=ARC_download_core,
#                                 args=(i, j, dic, type, len(events),
#                                       events, add_event, Sta_req,
#                                       input,))
#     par_jobs.append(p)
# sub_par_jobs = []
# for l in range(len(par_jobs)):
#     counter = input['req_np']
#     while counter >= input['req_np']:
#         counter = 0
#         for ll in range(len(sub_par_jobs)):
#             if par_jobs[sub_par_jobs[ll]].is_alive():
#                 counter += 1
#         if not counter == input['req_np']:
#             print 'counter: %s' % counter

#     par_jobs[l].start()
#     sub_par_jobs.append(l)
#     print 'length of sub_pat_jobs: %s' % len(sub_par_jobs)

# counter = input['req_np']
# while counter > 0:
#     counter = 0
#     for ll in range(len(par_jobs)):
#         if par_jobs[ll].is_alive():
#             counter += 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  utility_codes.py
#   Purpose:   Collection of helping scripts, utility codes
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
from datetime import datetime, timedelta
import fnmatch
import glob
try:
    from obspy.geodetics import locations2degrees
except Exception, e:
    from obspy.core.util import locations2degrees
from obspy.core import read
from obspy.taup import taup
import os
import pickle
import smtplib
import sys
import time

import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)

# ##################### header_printer ##################################


def header_printer():
    """
    Printing the welcome message
    :return:
    """
    os.system('clear')
    print 80*'-'
    print '\t\tobspyDMT (ObsPy Data Management Tool)\n'
    print '\tAutomatic tool for Downloading, Processing and Management'
    print '\t\t\tof Large Seismological Datasets\n'
    print ':copyright:'
    print 'The ObsPy Development Team (devs@obspy.org)\n'
    print 'Developed by Kasra Hosseini'
    print 'email: hosseini@geophysik.uni-muenchen.de\n'
    print ':license:'
    print 'GNU General Public License, Version 3'
    print '(http://www.gnu.org/licenses/gpl-3.0-standalone.html)'
    print 80*'-' + '\n'

# ##################### goodbye_printer ##################################


def goodbye_printer(input_dics, t1_pro):
    print "\n\n=================================================="
    print "obspyDMT main program has finished !\n"
    try:
        size = getFolderSize(input_dics['datapath'])
        size /= (1024.**2)
        print "Info:"
        print "* The storing directory contains %s MB of data." \
            % "{:.3f}".format(float(size))
        print input_dics['datapath']
        print "* Total time of execution: %s (h:m:s)" \
              % str(timedelta(seconds=round(float(time.time() - t1_pro))))
        print "==================================================\n\n"
    except Exception as e:
        print 'ERROR: %s' % e
        pass
# ##################### read_list_stas ##################################


def read_list_stas(add_list, normal_mode_syn, specfem3D):
    """
    read a list of stations instead of checking the availability.
    :param add_list:
    :param normal_mode_syn:
    :param specfem3D:
    :return:
    """

    print '\n----------------------------------------'
    print 'INFO:'
    print 'Format of the station list:'
    print 'sta  net  loc  cha  lat  lon  ele  depth'
    print '----------------------------------------\n\n'

    list_stas_fio = open(add_list)
    list_stas = list_stas_fio.readlines()
    for sta in range(len(list_stas)):
        if not list_stas[sta].startswith('\n'):
            list_stas[sta] = list_stas[sta].split()

    final_list = []
    if specfem3D == 'Y':
        for sta in range(len(list_stas)):
            for chan in ['MXE', 'MXN', 'MXZ']:
                final_list.append(['SY', list_stas[sta][0],
                                   'S3', chan,
                                   list_stas[sta][4],
                                   list_stas[sta][5],
                                   list_stas[sta][6]])
    elif normal_mode_syn == 'Y':
        for sta in range(len(list_stas)):
            for chan in ['LXE', 'LXN', 'LXZ']:
                final_list.append(['SY', list_stas[sta][0],
                                   'S1', chan,
                                   list_stas[sta][4],
                                   list_stas[sta][5],
                                   list_stas[sta][6]])
    else:
        for sta in range(len(list_stas)):
            # for chan in ['BH1', 'BH2', 'BHE', 'BHN', 'BHZ']:
            final_list.append([list_stas[sta][1], list_stas[sta][0],
                               list_stas[sta][2], list_stas[sta][3],
                               list_stas[sta][4], list_stas[sta][5],
                               list_stas[sta][6], list_stas[sta][7]])
    return final_list

# ##################### read_event_dic ##############################


def read_event_dic(address):
    """
    Reads event dictionary ("info" folder)
    :param address:
    :return:
    """
    if not os.path.isabs(address):
        address = os.path.abspath(address)

    if os.path.basename(address) == 'info':
        target_add = [address]
    elif locate(address, 'info'):
        target_add = locate(address, 'info')
    else:
        print 'Error: There is no "info" directory in %s' % address
        target_add = None

    if len(target_add) > 1:
        sys.exit('length of directories to read the event info: %s'
                 % len(target_add))

    event_dic = False
    for t_add in target_add:
        if os.path.isfile(os.path.join(t_add, 'event.pkl')):
            ev_file_fio = open(os.path.join(t_add, 'event.pkl'), 'r')
            event_dic = pickle.load(ev_file_fio)
        else:
            sys.exit('event.pkl can not be found in: %s' % t_add)
    return event_dic

# ##################### read_station_event ##############################


def read_station_event(address):
    """
    Reads the station_event file ("info" folder)
    :param address:
    :return:
    """
    if not os.path.isabs(address):
        address = os.path.abspath(address)

    if os.path.basename(address) == 'info':
        target_add = [address]
    elif locate(address, 'info'):
        target_add = locate(address, 'info')
    else:
        print 'Error: There is no "info" directory in %s' % address
        target_add = None

    sta_ev = []
    for t_add in target_add:
        if os.path.isfile(os.path.join(t_add, 'station_event')):
            sta_file_open = open(os.path.join(t_add, 'station_event'), 'r')
        else:
            print '====================================='
            print 'station_event could not be found'
            print 'Start Creating the station_event file'
            print '====================================='
            create_station_event(address=t_add)
            sta_file_open = open(os.path.join(t_add, 'station_event'), 'r')
        sta_file = sta_file_open.readlines()
        sta_ev_tmp = []
        for s_file in sta_file:
            sta_ev_tmp.append(s_file.split(','))
        sta_ev.append(sta_ev_tmp)
    return sta_ev

# ##################### create_station_event ############################


def create_station_event(address):
    """
    Creates the station_event file ("info" folder)
    :param address:
    :return:
    """

    event_address = os.path.dirname(address)
    if os.path.isdir(os.path.join(event_address, 'BH_RAW')):
        sta_address = os.path.join(event_address, 'BH_RAW')
    elif os.path.isdir(os.path.join(event_address, 'BH')):
        sta_address = os.path.join(event_address, 'BH')
    else:
        print 'ERROR: There is no reference (BH_RAW or BH) ' \
              'to create station_event file!'
        sys.exit()

    ls_stas = glob.glob(os.path.join(sta_address, '*.*.*.*'))
    ls_stas.sort()

    print '%s stations found in %s' % (len(ls_stas), sta_address)

    for i in range(len(ls_stas)):
        print i,
        sta_file_open = open(os.path.join(address, 'station_event'), 'a')
        try:
            sta = read(ls_stas[i])[0]
        except Exception as e:
            print 'WARNING: NOT readable: %s\n%s' % (ls_stas[i], e)
            sta = None
        try:
            sta_stats = sta.stats
            sta_info = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,iris,\n' \
                       % (sta_stats.network,
                          sta_stats.station,
                          sta_stats.location,
                          sta_stats.channel,
                          sta_stats.sac.stla,
                          sta_stats.sac.stlo,
                          sta_stats.sac.stel,
                          sta_stats.sac.stdp,
                          os.path.basename(event_address),
                          sta_stats.sac.evla,
                          sta_stats.sac.evlo,
                          sta_stats.sac.evdp,
                          sta_stats.sac.mag)
        except Exception as e:
            print '\nWARNING: Can not read all the required information ' \
                  'from the headers, some of them are presumed!'
            print e
            sta_info = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,iris,\n' \
                       % (sta_stats.network,
                          sta_stats.station,
                          sta_stats.location,
                          sta_stats.channel,
                          -12345.0, -12345.0, -12345.0, -12345.0,
                          os.path.basename(event_address),
                          -12345.0, -12345.0, -12345.0, -12345.0)
        sta_file_open.writelines(sta_info)
        sta_file_open.close()
    print 'station_event file is created in %s' % os.path.join(address,
                                                               'station_event')

# ##################### locate ##########################################


def locate(root='.', target='info'):
    """
    Locates a subdirectory within a directory.
    :param root:
    :param target:
    :return:
    """

    matches = []
    for root, dirnames, filenames in os.walk(root):
        for dirname in fnmatch.filter(dirnames, target):
            matches.append(os.path.join(root, dirname))

    return matches

# ##################### calculate_time_phase ##################################


def calculate_time_phase(event, sta):
    """
    calculate arrival time of the requested phase to use in retrieving
    waveforms.
    :param event:
    :param sta:
    :return:
    """

    ev_lat = event['latitude']
    ev_lon = event['longitude']
    ev_dp = abs(float(event['depth']))
    sta_lat = float(sta[4])
    sta_lon = float(sta[5])
    delta = locations2degrees(ev_lat, ev_lon, sta_lat, sta_lon)
    tt = taup.getTravelTimes(delta, ev_dp)
    phase_list = ['P', 'Pdiff', 'PKIKP']

    time_ph = 0
    flag = False
    for ph in phase_list:
        for i in range(len(tt)):
            if tt[i]['phase_name'] == ph:
                flag = True
                time_ph = tt[i]['time']
                break
            else:
                continue
        if flag:
            print 'Phase: %s' % ph
            break
    t_start = event['t1'] + time_ph
    t_end = event['t2'] + time_ph
    return t_start, t_end

# ##################### getFolderSize ###################################


def getFolderSize(folder):
    """
    Returns the size of a folder in bytes.
    :param folder:
    :return:
    """
    total_size = os.path.getsize(folder)
    for item in os.listdir(folder):
        itempath = os.path.join(folder, item)
        if os.path.isfile(itempath):
            total_size += os.path.getsize(itempath)
        elif os.path.isdir(itempath):
            total_size += getFolderSize(itempath)
    return total_size

# ##################### send_email ######################################


def send_email(input_dics):
    """
    Sending email to the specified "email" address
    :param input_dics:
    :return:
    """
    print '\n*********************************************'
    print 'Sending email to the following email-address:'
    print input_dics['email']
    print '*********************************************'
    t2_str = datetime.now()

    fromaddr = 'obspyDMT'
    toaddrs = input_dics['email']
    msg = "request finished at:\n%s" % t2_str

    try:
        server = smtplib.SMTP('localhost')
        server.sendmail(fromaddr, toaddrs, msg)
    except Exception as err:
        print '\nNo e-mail sent, as:\n>>:\t', err
        # err_info = traceback.extract_tb(sys.exc_info()[2])

# ##################### check_par_jobs ######################################


def check_par_jobs(jobs, sleep_time=1):
    """
    check whether all the parallel jobs are finished or not
    :param jobs:
    :param sleep_time:
    :return:
    """
    pp_flag = True
    while pp_flag:
        for proc in jobs:
            if proc.is_alive():
                time.sleep(sleep_time)
                pp_flag = True
                break
            else:
                pp_flag = False
    if not pp_flag:
        print '\n\nAll %s processes are finished...\n' % len(jobs)



from setuptools import setup
#from distutils.core import setup

setup(
    name="obspyDMT",
    version="0.9.9",
    description="obspyDMT: A Python Toolbox for Retrieving and Processing of Large Seismological Datasets",
    author="Kasra Hosseini",
    author_email="hosseini@geophysik.uni-muenchen.de",
    url="https://github.com/kasra-hosseini/obspyDMT",
    download_url="https://github.com/kasra-hosseini/obspyDMT.git",
    keywords=["obspyDMT", "ObsPy", "Seismology"],
    packages=["obspyDMT", "obspyDMT.utils"],
    requires=['matplotlib', 'numpy'],
    entry_points={
        'console_scripts': [
            'obspyDMT = obspyDMT.obspyDMT:main',
        ],
    },
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
    long_description="""\
obspyDMT (obspy Data Management Tool) is a command line tool for retrieving,
processing and management of large seismological datasets in a fully automatic
way which can be run in serial or in parallel.
"""
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  create_list_stas.py
#   Purpose:   simple code to create list_stas to be used by DMT
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.

# Added this line for python 2.5 compatibility
from obspy import UTCDateTime
from obspy.fdsn import Client
from plotting_tools import get_coordinates


# ########################## INPUT
req_client = "RESIF"
starttime = None
endtime = None
network = "YV"
station = "*"
location = '*'
channel = '*H*'

file_name = 'list_stas_created.txt'
# ########################## END INPUT

client = Client(req_client)
if starttime:
    starttime = UTCDateTime(starttime)
if endtime:
    endtime = UTCDateTime(endtime)
inv = client.get_stations(network=network, station=station,
                          location=location, channel=channel,
                          starttime=starttime, endtime=endtime,
                          level='channel')
content = inv.get_contents()
chans = list(set(content['channels']))
chans.sort()

net_inv = inv.networks[0]

fio = open(file_name, 'w')
for _i in range(len(chans)):
    net, sta, loc, cha = chans[_i].split('.')
    coord_chan = get_coordinates(net_inv, chans[_i], None)
    fio.writelines('%s  %s  %s  %s  %s  %s  %s  %s\n'
                   % (sta, net, loc, cha, coord_chan['latitude'],
                      coord_chan['longitude'], coord_chan['elevation'],
                      coord_chan['local_depth']))

fio.close()